import time

from selenium import webdriver
from time import sleep
from basement__.ContralerDatabase import Contraler_Database

class Crawler_Sougou:
    def __init__(self, chromeDriverPath=r'E:\Projects\webDriver\\chrome\\chromedriver.exe'):
        option = webdriver.ChromeOptions()
        option.add_experimental_option('excludeSwitches', ['enable-automation'])
        option.add_argument('--disable-blink-features=AutomationControlled')

        self.browser = webdriver.Chrome(executable_path=chromeDriverPath, options=option)
        self.contraler_db = Contraler_Database('paragraphdatabase')
        self.browser.get('https://weixin.sogou.com/weixin?type=2&s_from=input&query=%E8%82%A1%E7%A5%A8%E5%85%A5%E9%97%A8%E5%9F%BA%E7%A1%80%E7%9F%A5%E8%AF%86&ie=utf8&_sug_=y&_sug_type_=&w=01019900&sut=1323&sst0=1637824112380&lkt=0%2C0%2C0')
        sleep(5)

    def get_articleList(self):
        articleList = []
        liList = self.browser.find_elements_by_xpath('//ul[@class="news-list"]//li')
        for li in liList:
            pubTime = li.find_element_by_xpath('.//span[@class="s2"]').text
            url = li.find_element_by_xpath('.//h3//a').get_attribute('href')
            if('小时前' in pubTime):
                articleList.append(url)
        return articleList

    def get_content(self):
        sections = self.browser.find_elements_by_xpath('//div[@class="rich_media_content "]//section')
        paragraphList = []
        for section in sections:
            paragraph = section.text
            if(paragraph!='' and self.filter_unrelative(paragraph)):
                paragraphList.append(paragraph)
        return paragraphList

    def filter_unrelative(self, paragraph):
        filter_words = [
            '戳上面的', '专注分享股票', '文中内容仅为个人观点', '标签', '长按二维码', '扫码关注', '关注公众号', '添加助理', '扫描二维码','加助理进群',
            '好的投资理财软件', '关注我们', '入门学习', '记得置顶', '点我【关注】', '直播回放', '下期视频告诉你'
        ]
        for word in filter_words:
            if(word in paragraph):
                return False
            else:
                continue
        return True

    def insert_db(self, pList:list):
        for paragraph in pList:
            sql = 'INSERT INTO `paragraphdatabase`.`tb_relativeparagraph_sougouweixin` (`paragraph`) VALUES (\'{}\');'.format(paragraph)
            self.contraler_db.insertData2DB(sql)

    def run(self):
        urlList = self.get_articleList()
        for url in urlList:
            self.browser.get(url)
            time.sleep(2)
            pList = self.get_content()
            self.insert_db(pList=pList)
        self.browser.close()














from basement__.IsCheck import IsCheck_uchar
'''
    清洁器 基类
    所有定制化清洁器都继承自该类
'''
class Base_Cleaner:
    cleanWordList = []     # 清洗词列表，存放准备清洗掉的字符/字符串

    irrelevantWordList = ['图']     # 杂七杂八的无规律可循不相关的文字 如： 图9-5
    cleanWordList_headnum = []  # 序号列表

    # 通用的待组合清洗字符字典列表，如数字、字母等
    universal_cleanWordDict = {
        'alphabeticList':[
            'A','B','C','D','E','F','G','H'
        ],
        'numList1':['一','二','三','四','五','六','七','八','九','十'],
        'numList2':['第一','第二','第三','第四','第五','第六','第七','第八','第九','第十'],
    }

    # 清洗掉空格和结尾的空格
    @staticmethod
    def delSpace(paragraph:str):
        return paragraph.replace("\r", "").replace("\n", "").replace("\t", "").replace("\xa0", "").replace("\u3000", "")

    # 删除开头的序号 如 1. 2. 1、 2、 1， 2，
    @staticmethod
    def delHeaderNum(paragraph:str, cleanWordList:list):
        '''
        清除段落头尾数字
        :param paragraph: 待清洗的字符串
        :param cleanWordList: 待清除的字符串
        :return: 清洗后的字符串
        '''
        for cleanword in cleanWordList:
            if (paragraph.startswith(cleanword)):
                paragraph = paragraph[len(cleanword):]
                break
        return paragraph

    # 删除股票名字和股票代码
    @staticmethod
    def delStocksNameAndCode(paragraph, stocksNameCodeList):
        '''
        （用于段落字符串清洗）清除字符串中的股票名/股票代码
        :param paragraph:
        :param stocksNameCodeList:
        :return:
        '''
        for item in stocksNameCodeList:
            li = []
            li.append(item[0] + "（{}）".format(item[1]))
            li.append("（{}）".format(item[0]))
            li.append("（{}）".format(item[1]))
            li.append(item[0])
            li.append(item[1])
            for stock in li:
                paragraph = paragraph.replace(stock, "")  # 删除股票关键字（包括中文及代买关键字）
        return paragraph

    # 删除杂七杂八的无规律可循不相关的文字 如： 图9-5
    @staticmethod
    def delIrrelevantWord(paragraph, irrelevantWordList):
        # irrelevantWordList = ['图']
        for irrelevantWord in irrelevantWordList:
            paragraph = paragraph.replace(irrelevantWord, "")
        return paragraph

    @staticmethod
    def del_webTag(comment):
        '''
        (针对爬取的评论内容设计的方法) 清除评论内容中有标签内容的
        :param comment:
        :return:
        '''
        s = ''
        for m in comment.split('>'):
            s = s + m.split('<')[0]
        return s

    @staticmethod
    def del_repeatstr(s):
        """去除重复字符串中重复的(
            注意：该功能只针对中文内容和符号以及英文内容，对数字的重复不进行清洗
            )
        """
        cleaned_s = ''
        for i in range(len(s)):
            if (i < len(s) - 1):
                if(IsCheck_uchar.is_number(s[i])):
                    cleaned_s = cleaned_s + s[i]
                elif(s[i] != s[i + 1]):
                    cleaned_s = cleaned_s + s[i]
                else:
                    continue
            elif (i == len(s) - 1):
                cleaned_s = cleaned_s + s[i]
            else:
                break
        return cleaned_s

    def integratedOp(self, paragraph):
        '''
        集成操作方法 输出的结果为最终清洗完成的结果 又子类重写定制
        :param paragraph: 待清洗的字符串
        :return: paragraph: 经过一系列自定义清洗后的
        '''
        print('未设置集成方法')
        # # 清序号
        # temp = self.delHeaderNum(paragraph)
        # # 清空格
        # temp = self.delSpace(temp)
        # # 清序号
        # temp = self.delHeaderNum(temp)
        # # 清股票
        # # temp = self.delStocksNameAndCode(temp)
        # # 清不相关的文字
        # result = self.delIrrelevantWord(temp)
        # return result
        pass









import time

from selenium import webdriver
from time import sleep
import requests
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.common.by import By
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.keys import Keys
from ..Poster.Poster import VideoPoster
from ..Filter.Filter import videoFilter
from ..universalTools import tools
from ..DatabaserOperator import databaseOperator as dbOp
import packageDIY.globalTools.douyinCrack as douyinCrack
from ..Filter.Cleaner import Cleaner_Title

# 已有图片链接下载图片的方法
def downVideo(urlpath, name, dstDirPath):
    print("下载： ", urlpath)
    r = requests.get(urlpath, verify=False, timeout=30)
    video = r.content       #响应的二进制文件
    with open(dstDirPath + str(name) + '.mp4','wb') as f:     #二进制写入
        f.write(video)
    r.close()   # 关闭很重要，确保不要过多的连接

# --------------------------- 爬取抖音视频的类 ----------------------------------

class crawler_Douyin:
    def __init__(self, captchaPath, videoDirPath,chromeDriverPath=r'E:\Projects\webDriver\chrome\chromedriver.exe'):
        self.dboperator = dbOp.dbOperator(databaseName='postedurldatabase')
        self.filter = videoFilter(dirOriPath=videoDirPath)

        option = webdriver.ChromeOptions()
        option.add_experimental_option('excludeSwitches', ['enable-automation'])
        option.add_argument('--disable-blink-features=AutomationControlled')

        self.browser0 = webdriver.Chrome(executable_path=chromeDriverPath, options=option)
        self.browser1 = webdriver.Chrome(executable_path=chromeDriverPath, options=option)
        self.browser0.get('https://www.douyin.com')
        self.browser1.get('https://www.douyin.com')
        sleep(2)
        self.douyinCracker0 = douyinCrack.DouyinCrack(captchaDstDirPath=captchaPath)
        self.douyinCracker1 = douyinCrack.DouyinCrack(captchaDstDirPath=captchaPath)
        sleep(2)

        self.handleSlideCheck() # 滑块验证
        self.theNewestTitle = ''

        # 滑块验证参数
        self.captchaPath = captchaPath


    # 首次进入 move2BottomTimes 为向下滑动的次数 默认350
    def enterIndexDouyin(self, move2BottomTimes, douyinUrlIndex='https://www.douyin.com/search/%E8%82%A1%E7%A5%A8?publish_time=1&sort_type=2&source=normal_search&type=video'):
        self.browser0.execute_cdp_cmd('Page.addScriptToEvaluateOnNewDocument', {
            'source': 'Object.defneProperty(navigator, "webdriver", {get: () => undefined})'
        })
        self.browser0.get(douyinUrlIndex)
        # 滑块验证
        self.handleSlideCheck()

        # 2.通过浏览器向服务器发送URL请求
        self.browser0.get(douyinUrlIndex)
        # 滑块验证
        self.handleSlideCheck()

        # 等待某个元素是否出现
        WebDriverWait(self.browser0, 10).until(
            # EC.text_to_be_present_in_element((By.XPATH, ''))
            EC.presence_of_element_located((By.XPATH, "//ul[@class='fbe2b2b02040793723b452dc2de2b770-scss _924e252b5702097b657541b9e3b21448-scss']"))
        )
        self.handleSlideCheck()
        # 这里获取的ul用于判断
        try:
            ul = self.browser0.find_element_by_xpath("//ul[@class='fbe2b2b02040793723b452dc2de2b770-scss _924e252b5702097b657541b9e3b21448-scss']")
            liList = ul.find_elements_by_xpath("./li")
        except Exception as e:
            # 滑块验证
            self.handleSlideCheck()


        try:
            liFirstTitle = liList[0].find_element_by_xpath(".//a[@class='caa4fd3df2607e91340989a2e41628d8-scss a074d7a61356015feb31633ad4c45f49-scss _9c976841beef15a22bcd1540d1e84c02-scss']") # 获取标题
        except Exception as e:
            liFirstTitle = '空'
            # 滑块验证
            self.handleSlideCheck()

        liEffectiveList = []  # 可上传的视频信息列表
        # 判断是否有上一次爬取
        if(self.theNewestTitle!=''):
            # 说明有上一次爬取
            self.moveToBottom(move2BottomTimes)
            if(not(liFirstTitle == self.theNewestTitle)):
                # 说明有更新
                for li in liList:
                    # 每获取一个则下载上传一个
                    try:
                        a = li.find_element_by_xpath(".//a[@class='caa4fd3df2607e91340989a2e41628d8-scss a074d7a61356015feb31633ad4c45f49-scss _9c976841beef15a22bcd1540d1e84c02-scss']")
                    except Exception as e:
                        # 滑块验证
                        self.handleSlideCheck()
                        continue
                    try:
                        publishTime = li.find_element_by_xpath(".//span[@class='b32855717201aaabd3d83c162315ff0a-scss _5b7c9df52185835724fdb7f876969abd-scss']").text
                        timeLength = li.find_element_by_xpath(".//span[@class='d170ababc38fdbf760ca677dbaa9206a-scss']")
                    except Exception as e:
                        # 滑块验证
                        self.handleSlideCheck()
                    title = a.text
                    videoPageUrl = a.get_attribute("href")

                    # 当爬取到最新的那个则跳出循环
                    if(title == self.theNewestTitle):
                        break
                    if (timeLength.split(":")[0].startswith('0') and timeLength.split(":")[0].replace('0', '') != ''):
                        min = timeLength.split(":")[0].lstrip('0')
                    else:
                        min = '0'

                    if (int(min) >= 1 and int(min) <= 6):
                        liEffectiveList.append((title, videoPageUrl, publishTime))
                    else:
                        continue
                # 更新最新的那个数据
                self.setTheNewestCrawledInfo(
                    liList[0].find_element_by_xpath(
                        ".//a[@class='caa4fd3df2607e91340989a2e41628d8-scss a074d7a61356015feb31633ad4c45f49-scss _9c976841beef15a22bcd1540d1e84c02-scss']")
                )
            return liEffectiveList
        else:
            # 滑块验证
            self.handleSlideCheck()
            # 向下滚动
            self.moveToBottom(move2BottomTimes)

            ul = self.browser0.find_element_by_xpath("//ul[@class='fbe2b2b02040793723b452dc2de2b770-scss _924e252b5702097b657541b9e3b21448-scss']")
            liList = ul.find_elements_by_xpath("./li")
            self.liList = liList
            for li in liList:
                # 每获取一个则下载上传一个
                try:
                    a = li.find_element_by_xpath(".//a[@class='caa4fd3df2607e91340989a2e41628d8-scss a074d7a61356015feb31633ad4c45f49-scss _9c976841beef15a22bcd1540d1e84c02-scss']")
                except Exception as e:
                    continue
                timeLength = li.find_element_by_xpath(".//span[@class='d170ababc38fdbf760ca677dbaa9206a-scss']").text
                publishTime = li.find_element_by_xpath(".//span[@class='b32855717201aaabd3d83c162315ff0a-scss _5b7c9df52185835724fdb7f876969abd-scss']").text
                title = a.text
                videoPageUrl = a.get_attribute("href")
                if(timeLength.split(":")[0].startswith('0') and timeLength.split(":")[0].replace('0','')!=''):
                    min = timeLength.split(":")[0].lstrip('0')
                else:
                    min = '0'

                if (int(min) >= 1 and int(min) <= 6):
                    liEffectiveList.append((title, videoPageUrl, publishTime))
                else:
                    continue
                sleep(5)
            return liEffectiveList

    # 处理滑块验证的方法：1 直接关闭 2 滑动验证
    def handleSlideCheck(self):
        sleep(5)
        try:
            self.EffectiveCookies0 = self.douyinCracker0.handle_SlideCheck(self.browser0)
        except Exception as e:
            # print("browser0 无滑块出现")
            pass
        try:
            self.EffectiveCookies1 = self.douyinCracker1.handle_SlideCheck(self.browser1)
        except Exception as e:
            # print("browser1 无滑块出现")
            pass


    def getRealVideo(self, videoList_, videoDirPath, coverSavedPath):
        poster = VideoPoster(videoDirPath=videoDirPath, coverSavedPath=coverSavedPath)
        filter_video = videoFilter(dirOriPath=videoDirPath)
        if (videoList_):
            videoList = self.filter.filter_posted(videoList_) # 过滤掉上传过的视频
            videoList = tools.cleanRepeated(videoList)  # 去重
            # 过滤标题操作
            videoList = filter_video.filter_keywordFromTitle(videoList)
        else:
            # videoList = self.filter.filter_posted(self.enterIndexDouyin())
            # videoList = tools.cleanRepeated(videoList)  # 去重
            # # 过滤标题操作
            # videoList = filter_video.filter_keywordFromTitle(videoList)
            # self.postableList = videoList
            return None
        i = 1
        if(videoList!=''):
            print('当前可上传的视频数量： ', len(videoList))
            for item in videoList:
                print("当前处理的视频链接： ", item[1])
                # 2.通过浏览器向服务器发送URL请求
                self.browser1.get(item[1])
                # 滑块验证
                self.handleSlideCheck()
                self.browser1.get(item[1])
                sleep(5)
                # 说明出现了滑块验证处理一下
                self.handleSlideCheck()

                # 获取发布时间，判断发布时间是否是当天，是的话才进行下一步操作，不是的话跳出循环进入下一个循环
                pubTime = self.browser1.find_element_by_xpath("//span[@class='h7qyKhTd']").text.split("：")[1]
                pubTime = "".join(pubTime.split("-"))

                if(pubTime!=tools.getCurDate()):
                    # 判断不是当天的视频则跳过
                    continue
                # 获取准确的可下载的视频链接
                try:
                    videoUrl = self.browser1.find_element_by_xpath("//video//source").get_attribute("src")
                except Exception as e:
                    print('拿不到链接')
                    time.sleep(1)
                    try:
                        videoUrl = self.browser1.find_element_by_xpath("//video//source")
                        if(len(videoUrl)>1):
                            videoUrl = videoUrl[0].get_attribute("src")
                        else:
                            videoUrl = videoUrl.get_attribute("src")
                    except Exception as e:
                        continue

                # 抖音标题删除标签
                title = Cleaner_Title.clean_douyin_method2(title=item[0])
                if (title.replace(' ', '') == ''):
                    continue
                downVideo(urlpath=videoUrl, name=str(i), dstDirPath=videoDirPath)

                # 上传
                res = poster.post_videoSingle(str(i) + '.mp4', title0=title)
                print("上传视频: ", i, " 上传情况： ", res)
                # 更新上传过的数据库 postedurldatabase
                sql = "INSERT INTO `postedurldatabase`.`tb_video_posted` (`title`) VALUES ('{}');".format(
                    item[0]
                )
                self.dboperator.insertData2DB(sql=sql)
                i = i + 1
            return videoList
        else:
            return '无更新'

    # 设置上一次爬取最新的那个数据的信息
    def setTheNewestCrawledInfo(self, title):
        self.theNewestTitle = title

    # 下滑到最底部
    def moveToBottom(self, times=350):
        # 以当前位置为参照向下滚动
        # self.browser.execute_script('window.scrollBy(0 ,1000)') # 从当前位置向下滚动1000px
        for i in range(0, times):
            self.browser0.find_element_by_tag_name('body').send_keys(Keys.ARROW_DOWN)  # 在这里使用模拟的下方向键
            sleep(0.01)

    # 滑块验证 XX 滑块验证有点问题， 手动验证吧，自动化滑块验证还做不了
    # 遇到的问题，滑块验证采用拼图的形式，需要先检验待拼的位置坐标才行
    #   抖音的验证流程， 服务器发送一个cookie s_v_web_id:Value 到客户端，客户端进行滑块验证，验证成功的话发送一个成功的请求到服务器端，服务器端对 对应s_v_web_id  做一些调整（赋予权限），客户端就能正常浏览了
    #   s_v_web_id在一次会话结束时候失效
    def checkSlide(self, browser, x):
        sleep(2)
        # 实例化鼠标操作
        action = ActionChains(browser)
        # 按住滑块
        action.click_and_hold(browser.find_element_by_xpath("//*[@class='captcha_verify_img_slide react-draggable sc-VigVT ggNWOG']")).perform()
        for i in range(x):
            action.move_by_offset(1, 0)
        sleep(2)
        # 释放滑块
        action.release().perform()
        pass




from . import base_filter
from ..Cleaner import cleaner_paragraph
from basement__ import ContralerDatabase as dbOp

class Filter_Keyparagraph(base_filter.Base_Filter):
    def integratedOp(self, paragraphList, databaseName, tableName, tableName4Tag, tagRefSqlKind='id'):
        result = []
        cleanerInstance = cleaner_paragraph.Cleaner_Paragraph()
        dbOperator = dbOp.Contraler_Database(databaseName=databaseName)
        for item in paragraphList:
            # 筛选有标签的
            if (self.filter_hasTag_keyParagraph(item)):
                # 进行清洗操作
                item[1] = cleanerInstance.integratedOp(item[1])
                # 筛选判断 ： 1 字符串长度不在125-250之间；2 段落含有股票名或代码 3 段落包含日期关键词
                ##          但凡满足上面任何一个的段落筛选条件的段落都过滤掉
                check = (not self.filter_BetweenNumberOfWords(item[1],whichKind='keyParagraph')) or self.filter_hasStockCode(item[1], self.stocksNameCodeList) or self.filter_dateRef(item[1], self.dateRefList)
                if (check):
                    # 进入该判断条件说明对应段落无效跳过， 因此希望有效段落的check最终为false
                    continue
                if (tagRefSqlKind == 'id'):
                    sql = "SELECT `tag_origin` FROM " + databaseName + "." + tableName4Tag + " WHERE `id`=" + str(
                        item[2]) + ";"
                elif (tagRefSqlKind == 'url'):
                    sql = "SELECT `tag_origin` FROM " + databaseName + "." + tableName4Tag + " WHERE `url`='" + str(
                        item[2]) + "';"
                result.append(
                    (
                        item[1],  # 段落内容
                        dbOperator.getOneDataFromDB(sql)[0],  # 段落关键词
                    )
                )
            else:
                continue
        dbOperator.closeDb()
        del dbOperator
        return result


from basement__ import Timer

class Test_TaskTimer_AutoDealwithPost:
    timerConfig = {
        "beginTime": '00:01:00',  # 注意表示 一位数字的要0开头
        "endTime": '23:59:00',
        "taskExcuteDelta": 600,  # 任务间隔
        "timeExcuteDelta": 43200,  # 定时器间隔 每个一天运行一次
        "whichKind": 'articles',
        'databaseUser' : 'root',
        'databasePasswd' : 'root',
        "databaseName": 'articledatabase',  # 对应的数据库名
        "tableName": ['tb_article_toutiao_content'],  # 待处理的表名——存放图片url信息的表
    }
    tasktimer_autodealwithpost = Timer.TaskTimer_AutoDealwithPost(timerConfig=timerConfig)

    def test_timerTaskRun(self):
        self.tasktimer_autodealwithpost.timerTaskRun()


if(__name__=='__main__'):
    print("---------------测试头条文章自动化爬取的功能-------------")
    print("4 TaskTimer_AutoDealwithPost")
    test_adp = Test_TaskTimer_AutoDealwithPost()
    test_adp.test_timerTaskRun()





from basement__ import ContralerDatabase as dbOp
from ..Cleaner import cleaner_comment
'''
    过滤上传过的内容的类
'''
class Filter_Posted():
    def __init__(self):
        self.dbOperator = dbOp.Contraler_Database(databaseName='postedurldatabase')

    # paragraph 为待处理上传的数据
    def filterPosted(self, paragraph):
        paragraph = paragraph.strip()  # 清除一下左右空格
        sql = "SELECT * FROM `postedurldatabase`.`tb_article_posted` where `paragraph` = \'{}\';".format(paragraph)
        check = self.dbOperator.getOneDataFromDB(sql)
        if(check):
            # 上传过130 12
            return True
        else:
            return False

    def run(self, dataOriList):
        result = []
        for item in dataOriList:
            if(not self.filterPosted(paragraph=item[1])):
                # 没上传过，加入结果
                result.append(item)
        return result


class Filter_Posted_Comment():
    def __init__(self):
        self.dbOperator = dbOp.Contraler_Database(databaseName='postedurldatabase')
        self.cleaner = cleaner_comment.Cleaner_Comment()

    # comment 为待处理上传的数据
    def filterPosted(self, comment):
        comment = self.cleaner.integratedOp(comment)
        comment = comment.strip()  # 清除一下左右空格
        sql = "SELECT * FROM `postedurldatabase`.`tb_comment_posted` where `comment` = \'{}\';".format(comment)

        check = self.dbOperator.getOneDataFromDB(sql)
        if(check):
            return True
        else:
            return False

    def run(self, dataOriList):
        result = []
        for item in dataOriList:
            if(not self.filterPosted(comment=item[1])):
                # 没上传过，加入结果
                result.append(item)
        return result


import datetime
import os
from moviepy.editor import VideoFileClip
import cv2
from ..DatabaserOperator import databaseOperator as dbOp

class FileCheck():
    def get_filesize(self, filename):
        u"""
        获取文件大小（M: 兆）
        """
        file_byte = os.path.getsize(filename)
        return self.sizeConvert(file_byte)

    def get_file_times(self, filePath):
        u"""
        获取视频时长（s:秒）
        """
        clip = VideoFileClip(filePath)
        file_time = self.timeConvert(clip.duration)
        return file_time

    def sizeConvert(self, size):  # 单位换算
        K, M, G = 1024, 1024 ** 2, 1024 ** 3
        if size >= G:
            return str(size / G) + 'G Bytes'
        elif size >= M:
            return str(size / M) + 'M Bytes'
        elif size >= K:
            return str(size / K) + 'K Bytes'
        else:
            return str(size) + 'Bytes'

    def timeConvert(self, size):  # 单位换算
        M, H = 60, 60 ** 2
        if size < M:
            return str(size) + u'秒'
        if size < H:
            return u'%s分钟%s秒' % (int(size / M), int(size % M))
        else:
            hour = int(size / H)
            mine = int(size % H / M)
            second = int(size % H % M)
            tim_srt = u'%s小时%s分钟%s秒' % (hour, mine, second)
            return tim_srt


class videoFilter():
    def __init__(self, dirOriPath="E:\Projects\\4spideWeb\\tutorial\\videoDownload\\"):
        self.videoDirPath = dirOriPath  # 存放视频的目录路径
        self.videoNameList = os.listdir(dirOriPath)  # 获取目录下所有图片的名字
        self.checker = FileCheck()
        self.videoPathList = []
        self.dbOperator = dbOp.dbOperator(databaseName='postedurldatabase')
        self.filterwordList = [
            '早评', '午评', '午间点评', '点评', '午间短评', '评述',
            '昨夜', '昨天', '今日', '今天', '明天', '明日', '十年未来', '明年',

            '周一', '周二', '周三', '周四', '周五', '周六', '周日', '下周', '本周', '现状', '本月',
            '九月', '十月', '十一月', '十二月', '一月',
            '9月', '10月', '11月', '12月', '1月',
            '华为',
            '板块', '军工', '白酒', '医药', '新能源光伏', '白酒', '煤炭', '电力', '新能源', '科技股', '储能', '光伏', '锂电池概念', '光伏',
            '银行券商',
            '主力拉升', '走势', '大盘', '小幅拉升', '上涨行情', '拉升', '窄幅震荡', '高开低走', '拉伸', '涨停', '底部擒牛',
            '值得长期持有', '将迎来', '股价大涨', '走强', '反弹', '创历史新高', '看涨幅',
            '冲啊', '重温', '一键选股', '宣布', '难怪',
            '广告', '这种图形', '手机版', '实盘记录', '最新消息'
        ]
        # 股票名
        stocksnamecodeList = self.dbOperator.getAllDataFromDB(sql="SELECT `name` FROM stocksnamecode.tb_namecode;")
        for stock in stocksnamecodeList:
            self.filterwordList.append(stock[0])
        # 获取日期作为过滤关键词
        date_str = str(datetime.date.today()).split('-')
        year = date_str[0]
        month = date_str[1]
        day = date_str[2]
        self.filterwordList.append(year + month + day)
        self.filterwordList.append(month + day)


        def translate(day):
            if (int(day) < 10):
                return '0' + str(day)
            else:
                return str(day)

        self.filterwordList.append(month + str(translate(int(day) - 1)))
        self.filterwordList.append(month + str(translate(int(day) + 1)))
        self.filterwordList.append(month + '.' + day)
        self.filterwordList.append(month + '.' + str(int(day)))
        self.filterwordList.append(month + '.' + str(translate(int(day) - 1)))
        self.filterwordList.append(month + '.' + str(translate(int(day) + 1)))
        self.filterwordList.append(month + '-' + day)
        self.filterwordList.append(month + '-' + str(translate(int(day) - 1)))
        self.filterwordList.append(month + '月' + day)
        self.filterwordList.append(month + '/' + day)
        self.filterwordList.append(month + '/' + str(int(day)))
        self.filterwordList.append(month + '/' + str(translate(int(day) - 1)))
        self.filterwordList.append(day + '日')
        self.filterwordList.append(str(translate(int(day) - 1)) + '日')

        self.filterwordList_bilibili = [
            '第一集', '第七集', '第三课'
        ]
        for i in range(20):
            self.filterwordList_bilibili.append('（' + str(i) + '）')



    # 获取视频文件的封面
    def getCoverImg(self, videoPath, coverSavedPath='E:\\cur_Cover.jpg',frameNum=180):
        # frameNum 没有输入帧数，默认帧数为180
        cap = cv2.VideoCapture(videoPath)  # 读取视频文件
        cap.set(cv2.CAP_PROP_POS_FRAMES, float(frameNum))
        if(cap.isOpened()):  # 判断是否正常打开
            rval, frame = cap.read()
        cv2.imencode('.jpg', frame)[1].tofile(coverSavedPath)
        cover = cv2.imencode('.jpg', frame)[1]
        cap.release()
        return cover

    # 判断视频时间长度是否满足条件 2-5min
    def checkIfTimeLength(self, videoPath):
        timeLength = self.checker.get_file_times(videoPath)
        if('分钟' in timeLength):
            t = int(timeLength.split('分钟')[0])
            if(t>=1 and t<=6):
                # print(timeLength)
                return True
            else:
                # print(timeLength)
                return False
        else:
            # print(timeLength)
            return False

    def filter_time(self):
        filteredNameList = []
        for videoName in self.videoNameList:
            if(self.checkIfTimeLength(videoPath=self.videoDirPath + videoName)):
                filteredNameList.append(videoName)
            else:
                continue
        return filteredNameList

    # 过滤掉上传过的视频
    def filter_posted(self, urlList):
        # 从数据库获取上传过的数据
        postedList = self.dbOperator.getAllDataFromDB("SELECT title, videoUrl FROM `postedurldatabase`.`tb_video_posted`;")
        tempList = []
        if(postedList):
            for postedItem in postedList:
                for item in urlList:
                    if(item[0] == postedItem[0]):
                        tempList.append(item)
                    else:
                        continue
        if(tempList):
            # 待上传的列表中有已上传过的数据，清除上传过的数据
            for item in tempList:
                urlList.remove(item)
        return urlList

    # 快手对于是否当天发布的判断，输入为视频的发布时间 如 5小时前 2天前
    def checkIfCurDatePub_kuaishou(self, str_pub):
        if('小时' in str_pub):
            return True
        else:
            return False

    # 过滤标题关键词
    def filter_keywordFromTitle(self, videoInfoLis):
        fl_videoInfoList = []
        for videoInfo in videoInfoLis:
            check = False
            for keyword in self.filterwordList:
                if (keyword in videoInfo[0]):
                    check = True
            if (videoInfo not in fl_videoInfoList and not check):
                fl_videoInfoList.append(videoInfo)
        return fl_videoInfoList

    # 针对哔哩哔哩的过滤标题关键词
    def filter_keywordFromTitle4bilibili(self, videoInfoLis):
        fl_videoInfoList = []
        for videoInfo in videoInfoLis:
            check = False
            for keyword in self.filterwordList_bilibili:
                if (keyword in videoInfo[0]):
                    check = True
            if (videoInfo not in fl_videoInfoList and not check):
                fl_videoInfoList.append(videoInfo)
        return fl_videoInfoList


# 针对抖音的Filter
class douyinFilter(videoFilter):
    def __init__(self, dirOriPath):
        videoFilter.__init__(self, dirOriPath=dirOriPath)
        douyinFilterWordList = [
            '#搞笑', '#动漫', '#物价'
        ]
        self.filterwordList.extend(douyinFilterWordList)









import re
from ..DatabaserOperator import databaseOperator as dbOp
from ..Cleaner import Cleaner
'''
    输入： 元组
    输出： 判断结果 布尔值
    注意： 筛选这里根据数据库表结构的不同，筛选方法有所差异
        关键词段落 _keyParagraph
        关联段落 _relativeParagraph
'''
class Filter():
    def __init__(self):
        # 日期相关关键词
        self.dateRefList = [
            '周一', '周二', '周三', '周四','周五', '周六', '周日'
        ]
        # 从数据库获取股票名字和代码
        sql = "select name, code from `tb_namecode`"
        dbOperator = dbOp.dbOperator('stocksnamecode')
        self.stocksNameCodeList = dbOperator.getAllDataFromDB(sql)
        dbOperator.closeDb()
        del dbOperator


    # 输入 段落
    def filter_BetweenNumberOfWords(self, paragraph, whichKind):
        if(whichKind == 'relativeParagraph'):
            if (len(paragraph) > 125 and len(paragraph) < 250):
                return True
            else:
                return False
        elif(whichKind == 'keyParagraph'):
            if (len(paragraph) > 50 and len(paragraph) < 250):
                return True
            else:
                return False

    # 过滤含有股票关键字（中文及对应代码）的段落 输入为段落字符串 输出为布尔值
    def filter_hasStockCode(self, paragraph):
        for stock in self.stocksNameCodeList:
            if(stock[0] in paragraph or stock[1] in paragraph):
                # 含有股票对应名称和代码 返回True
                return True
            return False

    # 过滤掉日期相关的段落 输入为段落字符串 输出为布尔值
    def filter_dateRef(self, paragraph):
        # 利用正则表达式 匹配 ['5:45', '7/10/2017', '3月2日', '98年3月3日', '2019年8月1日', '10月5日']
        # \b 表示边界，加上的话表对应模式的字符串左右两边必须是空格（边界），否则匹配不到
        pattern = re.compile(r'\b\d{2}/\d{2}/\d{4}\b|\b\d{1,2}:\d{1,2}\b|\b\d{1,2}/\d{1,2}/\d{2,4}\b|\d{2,4}年\d{1,2}月\d{1,2}日|\d{1,2}月\d{1,2}日')  # 定义匹配模式
        for date in self.dateRefList:
            if (date in paragraph):
                # 含有日期相关的关键字 返回True
                return True
            elif(re.findall(pattern, paragraph)):
                return True
            return False

    # 输入 元组
    def filter_hasTag_keyParagraph(self, item):
        if (item[3] == 'False'):
            return False
        else:
            return True


    def integratedOp_keyParagraph(self, item):
        # 先判断是否有标签，有标签才进行后面的判断
        if(self.filter_hasTag_keyParagraph(item)):
            # 有标签 接着判断字数在120-250间
            return self.filter_Between120to250(item[1])
        else:
            # 无标签 输出False
            return False

    # -> 关联段落判断关键词
    def checkIfHasKeyword_relativeParagraph(self, paragraph, keyword):
        if(keyword in paragraph):
            # 关键字存在而且段落字数符合要求返回True
            return True
        else:
            return False


    # 输入的列表为从数据库获取的列表, 输出的列表为 最终过滤完成输出 可以上传的列表(指定好了格式)  -> 关键词段落的集成操作方法
    def integratedOp4List_keyParagraph(self, paragraphList, databaseName, tableName, tableName4Tag, tagRefSqlKind='id'):
        result = []
        cleanerInstance = Cleaner.Cleaner()
        dbOperator = dbOp.dbOperator(databaseName=databaseName)
        for item in paragraphList:
            # 筛选有标签的
            if (self.filter_hasTag_keyParagraph(item)):
                # 进行清洗操作
                item[1] = cleanerInstance.integratedOp(item[1])
                # 筛选判断 ： 1 字符串长度不在125-250之间；2 段落含有股票名或代码 3 段落包含日期关键词
                ##          但凡满足上面任何一个的段落筛选条件的段落都过滤掉
                check = (not self.filter_BetweenNumberOfWords(item[1], whichKind='keyParagraph')) or self.filter_hasStockCode(item[1]) or self.filter_dateRef(item[1])
                if(check):
                    # 进入该判断条件说明对应段落无效跳过， 因此希望有效段落的check最终为false
                    continue
                if(tagRefSqlKind=='id'):
                    sql = "SELECT `tag_origin` FROM " + databaseName + "." + tableName4Tag + " WHERE `id`=" + str(
                        item[2]) + ";"
                elif(tagRefSqlKind=='url'):
                    sql = "SELECT `tag_origin` FROM " + databaseName + "." + tableName4Tag + " WHERE `url`='" + str(
                        item[2]) + "';"
                result.append(
                    (
                        item[1],  # 段落内容
                        dbOperator.getOneDataFromDB(sql)[0],  # 段落关键词
                    )
                )
            else:
                continue
        dbOperator.closeDb()
        del dbOperator
        return result

    # -> 关联段落的集成操作方法
    def integratedOp4List_relativeParagraph(self, paragraphList, keywordList):
        result = []
        cleanerInstance = Cleaner.Cleaner()
        for item in paragraphList:
            # 1 进行清洗操作
            item[1] = cleanerInstance.integratedOp(item[1])
            for keyword in keywordList:
                # 2 根据字符串长度125-250间进行筛选
                if (not self.filter_BetweenNumberOfWords(item[1], whichKind='relativeParagraph')):
                    # 段落字数再125-250间，有用可传
                    continue
                # 3 对每个段落进行关键词筛选 若有关键词则跳出当前循环
                if (self.checkIfHasKeyword_relativeParagraph(item[1], keyword)):
                    # 4 根据筛选结果导入可上传的数据
                    result.append(
                        (
                            item[1],  # 段落内容
                            keyword,  # 相关关键词
                        )
                    )
                    break
        return result

class Filter_Posted():
    def __init__(self):
        self.dbOperator = dbOp.dbOperator(databaseName='postedurldatabase')

    # paragraph 为待处理上传的数据
    def filterPosted(self,paragraph):
        paragraph = paragraph.strip()  # 清除一下左右空格
        sql = "SELECT * FROM `postedurldatabase`.`tb_article_posted` where `paragraph` = \'{}\';".format(paragraph)

        check = self.dbOperator.getOneDataFromDB(sql)
        if(check):
            # 上传过130 12
            return True
        else:
            return False

    def run(self, dataOriList):
        result = []
        for item in dataOriList:
            if(not self.filterPosted(paragraph=item[1])):
                # 没上传过，加入结果
                result.append(item)
        return result



import time
from selenium.webdriver.common.keys import Keys


"""用于selenium的通用类"""
class Roll_To_Bottom:
    """滚动到底部的类"""
    @classmethod
    # 下滑到最底部
    def roll_tobottom_method1(cls, browser, times=350):
        '''
        向下滑动滚动条指定次数
        :param browser 指定浏览器引擎
        '''
        for i in range(0, times):
            browser.find_element_by_tag_name('body').send_keys(Keys.ARROW_DOWN)  # 在这里使用模拟的下方向键
            time.sleep(0.1)
        return None

    @classmethod
    def roll_tobottom_method2(cls, browser, distance):
        '''
        向下滑动滚动条指定px距离
        :param browser 指定浏览器引擎
        :param distance 指定像素距离
        '''
        # 以当前位置为参照向下滚动
        browser.execute_script('window.scrollBy(0 ,{})'.format(distance)) # 从当前位置向下滚动1000px
        return None

class Click_Element:
    def click_element(self, ele):
        ele.click()

class Close_Browser:
    """关闭浏览器对象的类"""
    @classmethod
    def close_browser(cls, browser):
        """关闭浏览器"""
        browser.close()



from PIL import Image
import os
'''
    将图片转换成缩略图
'''
# 注意这里目录路径的最后要加上 “//”
class Imgs2ThumbnailByDir():
    def __init__(self, thumbnailOriDirPath, thumbnailDstDirPath):
        self.thumbnailOriDirPath = thumbnailOriDirPath
        self.thumbnailDstDirPath = thumbnailDstDirPath
        self.thumbnailOriImgsList = os.listdir(thumbnailOriDirPath)

    def getCutOffWidthandHeight(self, widthOri, heightOri, scaleW=121, scaleH=75):
        widthCheck = 121/75 * heightOri
        i = 0
        while(widthCheck > widthOri):
            i = i + 1
            widthCheck = 121/75 * (heightOri-i)
        return (widthCheck,heightOri-i)

    # 等比例裁切做成缩略图 121：75 定宽
    def cutOff2ThumbnailSingle(self, imgName, srcDirPath, dstDirPath, scaleW=121, scaleH=75):
        imgSrc = srcDirPath + imgName
        print("正在处理的图： ", imgSrc)
        img = Image.open(imgSrc)
        if (img is not None):
            imgSize = img.size
            if (imgSize[1] < 500 and imgSize[0] > imgSize[1]):
                widthOri = imgSize[0]
                heightOri = imgSize[1]
                sizeNew = self.getCutOffWidthandHeight(widthOri=widthOri, heightOri=heightOri)
            elif (imgSize[1] < 500 and imgSize[0] < imgSize[1]):
                widthOri = imgSize[1]
                heightOri = imgSize[0]
                sizeNew = self.getCutOffWidthandHeight(widthOri=widthOri, heightOri=heightOri)
            elif (imgSize[1] >= 500 and imgSize[0] > imgSize[1]):
                # print("图片宽度超过500, 缩放为高度500")
                widthOri = int(imgSize[0] / imgSize[1] * 500)
                heightOri = 500
                img = img.resize((int((imgSize[0] / imgSize[1]) * 500), 500), Image.ANTIALIAS)  # 缩放
                sizeNew = self.getCutOffWidthandHeight(widthOri=widthOri, heightOri=heightOri)
                print(imgName, "  widthOri: ", widthOri, "  heightOri: ", heightOri, "  sizeNew: ", sizeNew)
            elif (imgSize[1] >= 500 and imgSize[0] < imgSize[1]):
                # print("图片宽度超过500, 缩放为高度500")
                widthOri = 500
                heightOri = int(imgSize[0] / imgSize[1] * 500)
                img = img.resize((int((imgSize[0] / imgSize[1]) * 500), 500), Image.ANTIALIAS)  # 缩放
                sizeNew = self.getCutOffWidthandHeight(widthOri=heightOri, heightOri=widthOri)
            else:
                print('整成缩略图有问题的：', imgSrc)
                img.save('E:\\imgs\\sthwrong\\' + imgName)
                return 0
            box = (0,0,sizeNew[0], sizeNew[1])
            img = img.crop(box)
            img.save(dstDirPath + imgName)

    # 对目录下的所有文件进行缩略图转换并放在缩略图目录下
    def cutOff2ThumbnailByDir(self):
        for imgName in self.thumbnailOriImgsList:
            self.cutOff2ThumbnailSingle(imgName=imgName, srcDirPath=self.thumbnailOriDirPath, dstDirPath=self.thumbnailDstDirPath, scaleW=121, scaleH=75)

class ImgsSetSize:
    def __init__(self, imgsOriDirPath):
        self.imgsOriDirPath = imgsOriDirPath
        self.oriImgNameList = os.listdir(imgsOriDirPath)

    # 获取图片大小
    def get_size(self, imgSrc):
        # 获取文件大小:KB
        size = os.path.getsize(imgSrc)
        return size / 1024

    # 压缩图片到指定大小，尺寸不变
    def compress_image(self, imgName, mb=150, step=1, quality=80, k=0.9):
        """不改变图片尺寸压缩到指定大小
        :param infile: 压缩源文件
        :param outfile: 压缩文件保存地址
        :param mb: 压缩目标，KB
        :param step: 每次调整的压缩比率
        :param quality: 初始压缩比率
        :return: 压缩文件地址，压缩文件大小
        """
        imgSrc = self.imgsOriDirPath + imgName
        o_size = self.get_size(imgSrc)
        if o_size <= mb:
            return imgSrc
        while o_size > mb:
            img = Image.open(imgSrc)
            x, y = img.size
            img = img.resize((int(x*k), int(y*k)), Image.ANTIALIAS)
            img.save(imgSrc, quality=quality)
            if quality - step < 0:
                break
            quality -= step
            o_size = self.get_size(imgSrc)
        return imgSrc, self.get_size(imgSrc)


    # 判断图片宽度是否超过制定长度默认1000，超过则按比例改变宽度为1000，保存原位置
    def resize_singleImg(self, imgName, setWidth=1000):
        imgOriSrc = self.imgsOriDirPath + imgName
        img = Image.open(imgOriSrc)
        if (img is not None):
            imgSize = img.size
            if(imgSize[0] > setWidth and imgSize[0] > imgSize[1]):
                # print("图片宽度超过500, 缩放为高度500")
                widthOri = setWidth
                heightOri = int(imgSize[1] / imgSize[0] * setWidth)
                img = img.resize((setWidth, heightOri), Image.ANTIALIAS)  # 缩放
                img.save(imgOriSrc)
            else:
                pass


    # 把指定目录下的图片resize，覆盖原文件
    def resize_byDir(self):
        for imgName in self.oriImgNameList:
            self.resize_singleImg(imgName, setWidth=1000)
            self.compress_image(imgName)




import hashlib, base64
from urllib import parse

"""
    加密相关的基类和方法
"""
# 加密类
class Encode:
    # 统一输出类型为str
    def bytes2str(self, b):
        return str(b, encoding='utf-8')

    def str2bytes(self, s):
        return bytes(s, encoding='utf-8')

    def encodeByMd5(self, s):
        return hashlib.md5(s.encode(encoding='utf-8')).hexdigest()

    # base64输出的为bytes类型 要转化为字符串
    def encodeByBase64(self, s):
        res = base64.encodebytes(s).strip()
        # 转换为字符串
        res = self.bytes2str(res)
        return res

    def encode0(self,s):
        return self.encodeByBase64(self.str2bytes(self.encodeByMd5(s)))

    @staticmethod
    def str2urlcode(s):
        '''
        将字符串转换为浏览器url编码格式
        '''
        return parse.quote(s)

    @staticmethod
    def urlcode2str(urlcode):
        '''
        将url编码转换为字符串格式
        '''
        return parse.unquote(urlcode)


from basement__.ContralSelenium import Base_Selenium
from selenium import webdriver
import string
import zipfile

class Contraler_Selenium(Base_Selenium):
    """定制Selenium特定功能"""
    def add_agent(self, proxyHost="u7125.5.tp.16yun.cn", proxyPort= "6445", proxyUser="16OINIUS", proxyPass= "971935", plugin_path_dir='E:/yiniuyun/', browser_driver_path="E:\Projects\webDriver\\chrome\\chromedriver.exe"):
        """
        创建添加爬虫代理的browser 此处采用的爬虫代理使用代码来源于 亿牛云
        """
        def create_proxy_auth_extension(proxyHost, proxyPort, proxyUser, proxyPass, scheme='http', plugin_path=None, plugin_path_dir=plugin_path_dir):
            if plugin_path is None:
                plugin_path = plugin_path_dir + 'tmp/{}_{}@t.16yun.zip'.format(proxyUser, proxyPass)

            manifest_json = """
                        {
                            "version": "1.0.0",
                            "manifest_version": 2,
                            "name": "16YUN Proxy",
                            "permissions": [
                                "proxy",
                                "tabs",
                                "unlimitedStorage",
                                "storage",
                                "<all_urls>",
                                "webRequest",
                                "webRequestBlocking"
                            ],
                            "background": {
                                "scripts": ["background.js"]
                            },
                            "minimum_chrome_version":"22.0.0"
                        }
                        """

            background_js = string.Template(
                """
                var config = {
                    mode: "fixed_servers",
                    rules: {
                        singleProxy: {
                            scheme: "${scheme}",
                            host: "${host}",
                            port: parseInt(${port})
                        },
                        bypassList: ["localhost"]
                    }
                  };

                chrome.proxy.settings.set({value: config, scope: "regular"}, function() {});

                function callbackFn(details) {
                    return {
                        authCredentials: {
                            username: "${username}",
                            password: "${password}"
                        }
                    };
                }

                chrome.webRequest.onAuthRequired.addListener(
                    callbackFn,
                    {urls: ["<all_urls>"]},
                    ['blocking']
                );
                """
            ).substitute(
                host=proxyHost,
                port=proxyPort,
                username=proxyUser,
                password=proxyPass,
                scheme=scheme,
            )
            with zipfile.ZipFile(plugin_path, 'w') as zp:
                zp.writestr("manifest.json", manifest_json)
                zp.writestr("background.js", background_js)
            return plugin_path

        proxy_auth_plugin_path = create_proxy_auth_extension(
            proxyHost=proxyHost, proxyPort=proxyPort, proxyUser=proxyUser, proxyPass=proxyPass
        )
        option = webdriver.ChromeOptions()
        # 如报错 chrome-extensions
        # option.add_argument("--disable-extensions")
        option.add_extension(proxy_auth_plugin_path)
        # 关闭webdriver的一些标志
        # option.add_experimental_option('excludeSwitches', ['enable-automation'])
        browser = webdriver.Chrome(
            chrome_options=option,
            executable_path=browser_driver_path
        )
        return browser

    def test_agent(self, browser):
        """测试爬虫代理ip 输出ip地址"""
        return browser.get('http://httpbin.org/ip')

    def run(self):
        self.browser = self.add_agent(proxyHost="u7125.5.tp.16yun.cn", proxyPort= "6445", proxyUser="16OINIUS", proxyPass= "971935", browser_driver_path="E:\Projects\webDriver\\chrome\\chromedriver.exe")


import re
"""
    字符处理相关的基类和方法
"""

class IsCheck_uchar:
    '''
        底层设计 —— 字符判断 输出均为布尔值
    '''
    @staticmethod
    def is_chinese(uchar):
        """判断一个unicode是否是汉字"""
        if(u'\u4e00' <= uchar <=u'\u9fa5'):
            return True
        else:
            return False

    @staticmethod
    def is_number(uchar):
        """判断一个unicode是否是数字"""
        if(u'\u0030' <= uchar <= u'\u0039'):
            return True
        else:
            return False

    @staticmethod
    def is_alphabet(uchar):
        """判断一个unicode是否是英文字母"""
        if(u'\u0041' <= uchar <= u'\u005a' or u'\u0061' <= uchar <= u'\u007a'):
            return True
        else:
            return False

    @staticmethod
    def is_other(uchar):
        """判断是否非汉字、非数字、非英文字符"""
        if(not (IsCheck_uchar.is_chinese(uchar) or IsCheck_uchar.is_number(uchar) or IsCheck_uchar.is_alphabet(uchar))):
            return True
        else:
            return False

    @staticmethod
    def all_isNumber(s):
        """验证字符串是否全部为数字"""
        p = re.compile('^[0-9]*$')
        result = p.match(s)
        if (result):
            # 说明全部位数字，返回1
            return True
        else:
            return False



from selenium import webdriver
import time
from .universal import u_selenium
from selenium.webdriver import Remote
from selenium.webdriver.chrome import options
from selenium.common.exceptions import InvalidArgumentException

# Selenium driver多进程调用的类
class ReuseChrome(Remote):
    def __init__(self, command_executor, session_id):
        self.r_session_id = session_id
        Remote.__init__(self, command_executor=command_executor, desired_capabilities={})

    def start_session(self, capabilities, browser_profile=None):
        """重写该方法"""
        if(not isinstance(capabilities, dict)):
            raise InvalidArgumentException("Capabilities must be a dictionary")
        if(browser_profile):
            if("moz:firefoxOptions" in capabilities):
                capabilities["moz:firefoxOptions"]["profile"] = browser_profile.encoded
            else:
                capabilities.update({"firefox_profile": browser_profile.encoded})
        self.capabilities = options.Options().to_capabilities()
        self.session_id = self.r_session_id
        self.w3c = False

# 通过selenium获取指定信息的类
class Base_Selenium(
    u_selenium.Roll_To_Bottom,
    u_selenium.Close_Browser,
    u_selenium.Click_Element
):
    def __init__(self, *driverPath):
        if(driverPath!=()):
            self.browser = self.init_webdriver(chromeDriverPath=driverPath)

    def init_webdriver(self, chromeDriverPath):
        """
        创建输出webdriver对象
        :chromeDriverPath 引擎的路径
        :option
        """
        option = webdriver.ChromeOptions()
        option.add_experimental_option('excludeSwitches', ['enable-automation'])
        option.add_argument('--disable-blink-features=AutomationControlled')
        browser = webdriver.Chrome(executable_path=chromeDriverPath, options=option)
        return browser

    def add_agent(self, proxyHost="u7125.5.tp.16yun.cn", proxyPort= "6445", proxyUser="16OINIUS", proxyPass= "971935", browser_driver_path="E:\Projects\webDriver\\chrome\\chromedriver.exe"):
        """创建添加爬虫代理的browser 根据不同代理源再customFunction__.Contraler_selenium.contraler_selenium 中重写"""
        pass

    def getCookies(self, browser, *url):
        '''
        类方法，以对象形式输出指定链接返回的cookies
        :param url: 待打开的链接
        :param browser: 浏览器引擎
        :return: cookies对象
        '''
        # 获取cookie
        dictCookies = browser.get_cookies()
        cookies = {}
        for item in dictCookies:
            key = item['name']
            cookies[str(key)] = str(item['value'])
        return cookies

    def del_all_cookies(self):
        self.browser.delete_all_cookies()

    def get_params(self, url, *browser):
        if(browser==()):
            self.browser.get(url)
            time.sleep(1)
            cookies = self.getCookies(url, self.browser)
            headers = {}
            return {
                'cookies': cookies,
                'headers': headers
            }
        else:
            browser.get(url)
            time.sleep(1)
            cookies = self.getCookies(url, browser)
            headers = {}
            return {
                'cookies': cookies,
                'headers': headers
            }

    def closeBrowser(self, *browser):
        '''
        关闭指定浏览器，若浏览器为None则关闭对象浏览器
        :param browser:
        :return: None
        '''
        if(browser==()):
            self.browser.close()
        else:
            browser.close()






# coding=utf8
from basement__.universal import u_javascript
import sys

print(sys.getdefaultencoding())
js_filepath = r'E:\Projects\auto_datahandler\resource\test_.js'
js_executor = u_javascript.Executor_Javascript.execute_jsfile(filepath=js_filepath)
print(u_javascript.Executor_Javascript.execute_jsfunc(js_executor, 'fa', u'参数'))


from basement__.ContralSelenium import Base_Selenium
from basement__.Download import Downloader



class Base_Cracker(Base_Selenium):
    '''（基于selenium）破解器基类 集成一些通用常用功能'''
    def __init__(self, *driverPath):
        if(driverPath!=()):
            # 显示继承父类
            Base_Selenium.__init__(self, driverPath[0])
            self.downloader = Downloader()  # 资源下载器
        else:
            self.downloader = Downloader()  # 资源下载器

    def enter_url(self, url):
        '''
        进入指定链接
        :param url: 注意需要加上协议 如：https://
        :return:
        '''
        self.browser.get(url)
        pass

    def del_All_cookies(self):
        '''
        删除本地保存的所有cookie，以便网站弹出验证框
        :return: none
        '''
        self.browser.delete_all_cookies()
        pass

    def get_Ori_allCookies(self):
        return self.browser.get_cookies()

    def get_Obj_allCookies(self):
        # 获取cookie
        dictCookies = self.browser.get_cookies()
        cookies = {}
        for item in dictCookies:
            key = item['name']
            cookies[str(key)] = str(item['value'])
        return cookies

    def get_URL(self, *browser):
        if(browser==()):
            curUrl = self.browser.current_url()
        else:
            curUrl = browser.browser.current_url()
        return curUrl




import json

from selenium import webdriver
from time import sleep
import pymysql, time, requests
from customFunction__.Poster.poster_video import Poster_Video as VideoPoster
from customFunction__.Filter.filter_video import KuaishouFilter
from basement__.ContralerDatabase import Contraler_Database
from fake_useragent import UserAgent
from basement__.Encode import Encode
from customFunction__.Cleaner import cleaner_title



# 已有链接下载的方法
def downVideo(urlpath, name, dstDirPath):
    # 获取当前日期
    r = requests.get(urlpath, verify=False)
    video = r.content       #响应的二进制文件
    print(name)
    with open(dstDirPath + str(name) + '.mp4','wb') as f:     #二进制写入
        f.write(video)
    r.close()   # 关闭很重要，确保不要过多的连接


class Crawler_Kuaishou_Base:
    def __init__(self, url="https://www.kuaishou.com/search/video?searchKey=%E8%82%A1"):
        self.filter = KuaishouFilter(dirOriPath="E:\Projects\\4spideWeb\\tutorial\\videoDownload\\")
        option = webdriver.ChromeOptions()
        option.add_experimental_option('excludeSwitches', ['enable-automation'])
        option.add_experimental_option('useAutomationExtension', False)
        self.browser = webdriver.Chrome(executable_path="E:\Projects\webDriver\\chrome\\chromedriver.exe",options=option)
        self.browser.get(url)
        self.url = url
        self.contraler_db = Contraler_Database('videodatabase')
        # 登陆验证的
        self.browser.add_cookie({
            "domain": ".kuaishou.com",
            "name": "userId",
            "value": "2560728971",
            "path": '/',
            "expires": None
        })
        # 滑块验证的
        self.browser.add_cookie({
            "domain": ".kuaishou.com",
            "name": "did",
            "value": "web_dd4765fe13d667c461fd6269a10ae0df",
            "path": '/',
            "expires": None
        })
        self.browser.add_cookie({
            "domain": ".www.kuaishou.com",
            "name": "kpf",
            "value": "PC_WEB",
            "path": '/',
            "expires": None
        })
        self.browser.add_cookie({
            "domain": ".www.kuaishou.com",
            "name": "kpn",
            "value": "KUAISHOU_VISION",
            "path": '/',
            "expires": None
        })
        self.browser.add_cookie({
            "domain": ".kuaishou.com",
            "name": "clientid",
            "value": "3",
            "path": '/',
            "expires": None
        })
        self.browser.add_cookie({
            "domain": ".kuaishou.com",
            "name": "client_key",
            "value": "65890b29",
            "path": '/',
            "expires": None
        })
        self.browser.add_cookie({
            "domain": ".kuaishou.com",
            "name": "kuaishou.server.web_ph",
            "value": "f026d92ae2b0265bf86ec666fa786f635d5b",
            "path": '/',
            "expires": None
        })
        self.browser.add_cookie({
            "domain": ".kuaishou.com",
            "name": "kuaishou.server.web_st",
            "value": "ChZrdWFpc2hvdS5zZXJ2ZXIud2ViLnN0EqABWHDql4qaYN0DtvJlDUI2vX9v2hDjf2eJUChEjJHM9YZiNRQRhDkcURY8NFHcPTyo3KaaN9tV-q9kFdDhaa6sLjoDSmhK5UjN2byF-eFVit8lfNtBhKzJ-dtvFTmsotI8eHddsdCisokwNMCBCXzdiwA7fmnM7SKqP3qEsXkJsOZJlJ1-CAUQSCJwVaVaU3SvYCeGb3Vi9BVfw0RtpgCzexoS8JByODRPv5hk-B95zTquvFHcIiCS6UA7jHPQUTMshtZGa63oO8ZPkqrI5c9ylvllsQswiCgFMAE",
            "path": '/',
            "expires": None
        })
        self.poster = VideoPoster(videoDirPath='E:\\test\\', coverSavedPath='E:\\test_kuaishou.png')
        print("请先登陆")

    def getCookies(self, browser, *url):
        '''
        类方法，以对象形式输出指定链接返回的cookies
        :param url: 待打开的链接
        :param browser: 浏览器引擎
        :return: cookies对象
        '''
        # 获取cookie
        dictCookies = browser.get_cookies()
        cookies = {}
        for item in dictCookies:
            key = item['name']
            cookies[str(key)] = str(item['value'])
        return cookies

    def pull_down(self):
        """下拉到底"""
        # 下拉到底
        # js = "var q=document.documentElement.scrollTop=20000"
        self.browser.execute_script("""
                        (function () {
                        var y = 0;
                        var step = 100;
                        window.scroll(0, 0);
                        function f() {
                        if (y < document.body.scrollHeight) {
                        y += step;
                        window.scroll(0, y);
                        setTimeout(f, 100);
                        } else {
                        window.scroll(0, 0);
                            document.title += "scroll-done";
                        }
                        }
                        setTimeout(f, 1000);
                        })();
                        """)
        print("下拉中...")
        time.sleep(2)
        # while (True):
        #     if ("scroll-done" in self.browser.title):
        #         break
        #     else:
        #         print("还没有拉到最底端...")
        #         sleep(3)
        n = 0
        while (n <= 10):
            if ("scroll-done" in self.browser.title):
                break
            else:
                print("还没有拉到最底端...")
                sleep(3)
            n = n + 1

    # 提取视频名字的方法
    def videoNameExtract(self, videoSrc):
        return str(videoSrc).split("?")[0].split("/")[-1].replace('.mp4', '')


class Crawler_Kuaishou_newest(Crawler_Kuaishou_Base):
    def update_db(self, lis, searchword):
        for item in lis:
            authorid = item['author']['id']
            author_index = 'https://www.kuaishou.com/profile/' + authorid
            authorname = item['author']['name']
            sql = 'INSERT INTO `videodatabase`.`tb_kuaishou_author` (`name`, `index_url`, `seardword`) VALUES (\'{}\', \'{}\', \'{}\');'.format(
                authorname, author_index, searchword
            )
            if(self.contraler_db.getOneDataFromDB('SELECT * FROM `videodatabase`.`tb_kuaishou_author` WHERE `index_url`=\'{}\''.format(author_index))):
                continue
            self.contraler_db.insertData2DB(sql)


    def get_author_from_api(self, searchword, pcursor, url_api):
        searchword_urlcode = Encode.str2urlcode(searchword)
        cookies = self.getCookies(self.browser)
        headers = headers = {
            'Host': 'www.kuaishou.com',
            'Connection': 'keep-alive',
            'sec-ch-ua': '" Not;A Brand";v="99", "Google Chrome";v="91", "Chromium";v="91"',
            'accept': '*/*',
            'sec-ch-ua-mobile': '?0',
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36',
            'content-type': 'application/json',
            'Origin': 'https://www.kuaishou.com',
            'Sec-Fetch-Site': 'same-origin',
            'Sec-Fetch-Mode': 'cors',
            'Sec-Fetch-Dest': 'empty',
            'Accept-Encoding': 'gzip, deflate, br',
            'Accept-Language': 'zh-CN,zh;q=0.9'
        }
        headers['Referer'] = 'https://www.kuaishou.com/search/video?searchKey={}'.format(searchword_urlcode)
        headers['User-Agent'] = UserAgent().random
        data = {
            "operationName": "visionSearchPhoto",
            "variables": {
                "keyword": "股",
                "pcursor": "",
                "page": "search"
            },
            "query": """query visionSearchPhoto($keyword: String, $pcursor: String, $searchSessionId: String, $page: String, $webPageArea: String) {
        					visionSearchPhoto(keyword: $keyword, pcursor: $pcursor, searchSessionId: $searchSessionId, page: $page, webPageArea: $webPageArea) {
        						result
        						llsid
        						webPageArea
        						feeds {
        							type
        							author {
        								id
        								name
        								following
        								headerUrl
        								headerUrls {
        									cdn
        									url
        									__typename
        								}
        								__typename
        							}
        							tags {
        								type
        								name
        								__typename
        							}
        							photo {
        								id
        								duration
        								caption
        								likeCount
        								realLikeCount
        								coverUrl
        								photoUrl
        								liked
        								timestamp
        								expTag
        								coverUrls {
        									cdn
        									url
        									__typename
        								}
        								photoUrls {
        									cdn
        									url
        									__typename
        								}
        								animatedCoverUrl
        								stereoType
        								videoRatio
        								__typename
        							}
        							canAddComment
        							currentPcursor
        							llsid
        							status
        							__typename
        						}
        						searchSessionId
        						pcursor
        						aladdinBanner {
        							imgUrl
        							link
        							__typename
        						}
        						__typename
        					}
        				}"""
        }
        # 两次请求页面
        # 关键参数的配置
        data['variables']['keyword'] = searchword
        data['variables']['pcursor'] = str(pcursor)
        data_json = json.dumps(data)
        res = requests.post(url=url_api, headers=headers, cookies=cookies, data=data_json)
        return res


    def get_all_authorId(self, searchword, url_api):
        res0 = self.get_author_from_api(searchword=searchword, pcursor='',url_api=url_api)
        authors0 = res0.json()['data']['visionSearchPhoto']['feeds']
        # 关键参数的配置
        res1 = self.get_author_from_api(searchword=searchword, pcursor='1',url_api=url_api)
        authors1 = res1.json()['data']['visionSearchPhoto']['feeds']
        self.update_db(authors0, searchword=searchword)
        self.update_db(authors1, searchword=searchword)

        authorId_list = []
        for item in authors0:
            if(item not in authorId_list):
                authorId_list.append(item['author']['id'])

        for item in authors1:
            if (item not in authorId_list):
                authorId_list.append(item['author']['id'])
        return authorId_list

    # 获取clientCacheKey
    def get_videoPageUrlList(self, srcList):
        clientCacheKeyList = []
        videoPageUrlList = []
        for src in srcList:
            if ('clientCacheKey' in src):
                clientCacheKey = str(src).split("//")[1].split("clientCacheKey=")[1].split(".")[0]
                if (clientCacheKey not in clientCacheKeyList):
                    # 插入数据库操作
                    sql = "INSERT INTO `videodatabase`.`tb_kuaishou_resource` (`clientCacheKey`) VALUES (\'{}\');".format(clientCacheKey)
                    self.contraler_db.insertData2DB(sql=sql)
                    clientCacheKeyList.append(clientCacheKey)
            else:
                continue
        for item in clientCacheKeyList:
            videoPageUrl = 'https://www.kuaishou.com/short-video/' + item
            videoPageUrlList.append(videoPageUrl)
        return videoPageUrlList

    # 已经获取了clientCacheKey，下面这个方法已经不用了
    def crawlImgSrc(self, url):
        self.browser.get(url)
        sleep(3)
        # self.pull_down()    # 下拉
        videoImgList = self.browser.find_elements_by_xpath("//div[@class='video-card-main']//div[@class='card-link']/div[@class='poster']/img[@class='poster-img']")
        # 只看前10个视频
        videoImgList = videoImgList[:6]
        clientCacheListOrigin = []
        for item in videoImgList:
            clientCacheListOrigin.append(item.get_attribute('src'))
        return clientCacheListOrigin


    def getVideoListInfoByList(self,lis):
        videosInfoList = []
        for src in lis:
            videoInfo = self.getSingleVideoInfo(src)
            if(videoInfo):
                # 进入这里说明是视频是当天发布的，可以下载
                sql = "INSERT INTO `videodatabase`.`tb_kuaishou_videoinfo` (`title`, `author`, `time`, `videoSrc`, `videoName`) VALUES (\'{}\',\'{}\',\'{}\',\'{}\',\'{}\');".format(
                    videoInfo["videoTitle"],
                    videoInfo["author"],
                    videoInfo["publishTime"],
                    videoInfo["videoSrc"],
                    videoInfo["videoName"]
                )
                self.contraler_db.insertData2DB(sql)
                videosInfoList.append(videoInfo)
        return videosInfoList

    # 获取视频
    def getSingleVideoInfo(self, src):
        self.browser.get(src)
        sleep(4)
        # 获取视频信息
        try:
            # publishTime = self.browser.find_element_by_xpath("//div[@class='short-video-info']//div[@class='title-warp']//span[@class='photo-time']").text
            publishTime = self.browser.find_element_by_xpath("//span[@class='photo-time']").text
        except Exception as e:
            # publishTime = '小时'
            print(src, '有问题')

        videoInfo = {}
        if (self.filter.checkIfCurDatePub_kuaishou(publishTime)):
            time.sleep(2)
            # 进入这里说明是当天发布的
            author = self.browser.find_element_by_xpath(
                "//div[@class='short-video-info']//div[@class='title-warp']//span[@class='profile-user-name-title']").text
            videoTitle = self.browser.find_element_by_xpath("//div[@class='short-video-info-container-detail']/p").text
            videoSrc = self.browser.find_element_by_xpath("//video").get_attribute("src")
            videoName = self.videoNameExtract(videoSrc)
            videoInfo = {
                "author": author,
                "publishTime": publishTime,
                "videoTitle": videoTitle,
                "videoSrc": videoSrc,
                "videoName": videoName
            }
        return videoInfo

    def run(self, searchword='股票'):
        # 1 爬取搜索关键词视频下所有作者的id
        authorId_list = self.get_all_authorId(searchword=searchword, url_api='https://www.kuaishou.com/graphql')
        print("总共有作者数： ", len(authorId_list))
        self.authorId_list = authorId_list
        # 2 for循环 进入每个作者的首页，获取当天视频
        for authorId in authorId_list:
            author_index = 'https://www.kuaishou.com/profile/' + authorId
            clientCacheListOrigin = self.crawlImgSrc(author_index)
            # 处理爬取到的Src
            videoPageUrlList = self.get_videoPageUrlList(clientCacheListOrigin)
            videosInfoList = self.getVideoListInfoByList(videoPageUrlList)
            for videoInfo in videosInfoList:
                # 下载视频并上传
                downVideo(urlpath=videoInfo['videoSrc'], name=videoInfo['videoName'], dstDirPath='E:\\test\\')
                if(self.filter.filter_keyword_title(videoInfo['videoTitle'])):
                    # 说明含有相关过滤词，跳过
                    print("标题含过滤词，过滤： ", videoInfo)
                    continue
                # 清洗标题
                videoInfo["videoTitle"] = cleaner_title.Cleaner_Title.clean_douyin_method2(videoInfo["videoTitle"])
                if(videoInfo["videoTitle"].replace(' ','')==''):
                    continue
                res = self.poster.post_videoSingle(videoInfo['videoName'] + '.mp4', title0=videoInfo["videoTitle"])
        self.browser.close()
        return authorId_list



'''
    自动执行脚本
        根据参数选择对应的执行引擎
'''
import subprocess
import os
import datetime
import threading
import inspect, time
import ctypes
from . import run_auto_keyParagraph as rakp
from . import run_auto_relativeParagraph as rarp
from . import run_auto_contentImgs as raci
from . import run_auto_thumbnailImgs as rati
from . import run_auto_comment as rac
from . import run_auto_selenium as ras
from .articlesRef.DatabaserOperator import databaseOperator as dbOp
from . import run_auto_video as rav
from .spider_selenium import spider_sougou

# 1. 定时任务基类 (完成)
class TimedTaskBasic():
    '''
        定时任务的类
            taskExcuteDelta     任务的执行间隔 比如 8点到9点间每隔几秒执行一次任务
            timeExcuteDelta=86400     定时器的执行间隔 比如 每个一周运行一次定时器 86400为一天
            beginTime="09:00:00"
            endTime="10:00:00"
            taskExcuteDelta=30
            timeExcuteDelta=86400
    '''
    def __init__(self, setting):
        self.setting = setting  # 定时任务配置文件
        self.beginTime = self.setting['beginTime']  # 定时任务每天的启动时间 例子早上8点 08:00:00
        self.endTime = self.setting['endTime']  # 定时任务执行在几点之前 例子早上9点 09:00:00
        self.taskExcuteDelta = self.setting['taskExcuteDelta']  # 定时任务间隔几秒执行一次
        self.timeExcuteDelta = self.setting['timeExcuteDelta']

    #任务描述：每天早上8点，隔3600秒执行一次task任务
    def func1(self):
        '''早上8点开启任务'''
        self.func2()
        # 每隔一天（86400秒）执行一次func1
        t = threading.Timer(self.timeExcuteDelta,self.func1)
        t.start()

    def func2(self):
        '''8点-9点，隔3600秒执行一次任务task'''
        self.task()
        # 10点函数结束执行
        now_time = datetime.datetime.now()
        today_9 = datetime.datetime.strptime(str(datetime.datetime.now().year)+'-'+str(datetime.datetime.now().month)+'-'+str(datetime.datetime.now().day) + ' ' + str(self.endTime) ,'%Y-%m-%d %H:%M:%S')
        # 因为定时任务会延后10秒钟执行，所以设置终止条件时，需要提前10秒钟
        if(now_time <= today_9-datetime.timedelta(seconds=10)):
            print("当前时间 " + str(now_time) + " ,该时间在任务执行时段内，继续间隔 " + str(self.taskExcuteDelta)  + " 执行任务")
            t = threading.Timer(self.taskExcuteDelta, self.func2)
            t.start()
        else:
            print("当前时间 " + str(now_time) + " ,该时间不在任务执行时段内，等待 " + str(self.timeExcuteDelta) + " 后再重新开启执行任务定时器")

    def task(self):
        '''需要执行的任务'''
        print(' ---------- task任务开始执行 ---------- ')
        print("本定时器未设置定时任务内容")
        time.sleep(1)
        print(' ---------- task任务执行完毕 ---------- ')

    # 任务自动执行启动
    def timedTaskRun(self):
        #获取当前时间
        now_time = datetime.datetime.now()

        #获取当前时间年、月、日
        now_year = now_time.year
        now_month = now_time.month
        now_day = now_time.day

        # 今天早上8点时间表示
        # today_8 = datetime.datetime.strptime(str(now_year)+'-'+str(now_month)+'-'+str(now_day)+' '+'08:00:00','%Y-%m-%d %H:%M:%S')
        today_8 = datetime.datetime.strptime(str(now_year)+'-'+str(now_month)+'-'+str(now_day)+' ' + str(self.beginTime),'%Y-%m-%d %H:%M:%S')
        #明天早上8点时间表示
        tomorrow_8 = datetime.datetime.strptime(str(now_year)+'-'+str(now_month)+'-'+str(now_day)+' ' + str(self.beginTime),'%Y-%m-%d %H:%M:%S')

        # 判断当前时间是否过了今天凌晨8点,如果没过，则今天早上8点开始执行，过了则从明天早上8点开始执行，计算程序等待执行的时间
        if(now_time <= today_8):
            wait_time = (today_8 - now_time).total_seconds()
        else:
            wait_time = (tomorrow_8 - now_time).total_seconds()

        # 等待wait_time秒后（今天早上8点或明天早上8点），开启线程去执行func函数
        self.thread = threading.Timer(wait_time,self.func1)    # 当前线程
        self.thread.start()

# 永久执行任务
class PermanentTask(TimedTaskBasic):
    # 任务描述：永久任务 隔3600秒执行一次task任务
    def func1(self):
        # 该任务设置的
        #   beginTime="00:01:00"
        #   endTime="23:59:00"
        self.func2()


# 2. 定时任务 ———— 爬取数据的的类
class TimedTask4Spider(TimedTaskBasic):
    def task(self):
        '''
        setting = {
            "spiderPath": "E:\\Projects\\Crawl_Dealwith_Post_Auto\\stocksNameCode_Crawl_Auto\\stocksNameCode_Crawl_Auto",
            "spiderName": "StcockNameCodeSpider",
            "command": "Scrapy crawl StcockNameCodeSpider",
            "beginTime": '08:00:00',  # 注意表示 一位数字的要0开头
            "endTime": '09:00:00',
            "excuteDelta": 3600  # 间隔1h 也就是说一天执行一次
        }
        '''

        if (self.setting['tasktype'] == 'scrapy'):
            # 通过os改变工作路径,注意路径是绝对路径，而且还要是\\  只要环境是同一个 这样可以执行对应的爬虫项目
            os.chdir(self.setting['spiderPath'])
            subprocess.Popen(self.setting['command'])
        elif (self.setting['tasktype'] == 'selenium'):

            spider_sougou.spider_sougou(self.setting["origin"])

        else:
            print('tasktype参数出错')


# 结束线程的类没有用到
class StopThread():
    '''结束线程的类'''
    def _async_raise(self,tid, exctype):
        """raises the exception, performs cleanup if needed"""
        tid = ctypes.c_long(tid)
        if(not inspect.isclass(exctype)):
            exctype = type(exctype)
        res = ctypes.pythonapi.PyThreadState_SetAsyncExc(tid, ctypes.py_object(exctype))
        if(res == 0):
            raise ValueError("invalid thread id")
        elif(res != 1):
            # """if it returns a number greater than one, you're in trouble,
            # and you should call it again with exc=NULL to revert the effect"""
            ctypes.pythonapi.PyThreadState_SetAsyncExc(tid, None)
            raise SystemError("PyThreadState_SetAsyncExc failed")

    # 任务自动执行结束 结束本线程实例，在调用本方法之前得先调用 timedTaskRun()
    def stopCurThread(self, thread):
        self._async_raise(thread.ident, SystemExit)


# 2. 定时任务 ———— 处理数据及上传 段落数据 到接口的类（完成）
class TimedTask4AutoDealwithPost(TimedTaskBasic):
    def task(self):
        '''
        setting = {
            "beginTime": '08:00:00',  # 注意表示 一位数字的要0开头
            "endTime": '09:00:00',
            "excuteDelta": 3600,  # 间隔1h 也就是说一天执行一次
            "whichKind" : 'keyParagraph'
        }
        '''
        if(self.setting["whichKind"] == 'keyParagraph'):
            rakp.run(setting=self.setting)
        elif(self.setting["whichKind"] == 'relativeParagraph'):
            rarp.run(setting=self.setting)
        elif(self.setting["whichKind"] == 'contentImgs'):
            raci.run(proj_absPath=self.setting["proj_absPath"], oriDomain=self.setting["oriDomain"], database=self.setting['databaseName'], tableNameList=self.setting['tableName'], maskFilt=self.setting['maskFilt'])
        elif(self.setting["whichKind"] == 'thumbnailImgs'):
            rati.run(proj_absPath=self.setting["proj_absPath"], oriDomain=self.setting["oriDomain"], database=self.setting['databaseName'], tableNameList=self.setting['tableName'])
        elif (self.setting['whichKind'] == 'video' and self.setting['crawlMethod'] == 'Selenium'):
            ras.run_douyin(proj_absPath=self.setting["proj_absPath"], crawlUrl_list=self.setting['crawlUrl_list'],oriDomain=self.setting["oriDomain"])
        elif(self.setting["whichKind"]=='video' and self.setting['crawlMethod'] == 'Scrapy'):
            rav.run_bilibili(setting=self.setting)
        elif(self.setting["whichKind"]=='articleComment'):
            rac.run(setting=self.setting)
        print("上传数据完成，接下来清空数据库")
        # 上传完直接清空数据库，不用再用定时器
        clearDB = TimedTask4AutoClearDB(setting=self.setting)
        clearDB.task()


# 3. 定时任务 ———— 数据库对应上传过的数据清除的类
class TimedTask4AutoClearDB(TimedTaskBasic):
    def task(self):
        '''
        setting = {
            "beginTime": '08:00:00',  # 注意表示 一位数字的要0开头
            "endTime": '09:00:00',
            "excuteDelta": 3600,  # 间隔1h 也就是说一天执行一次
            "whichKind": 'keyParagraph',
            "databaseName" : ['stocksnamecode'],   # 对应的数据库名列表
            "tableName" : ['tb_namecode']    # 待清空的表名
        }
        '''
        if(self.setting['whichKind'] == 'keyParagraph' or self.setting['whichKind'] == 'relativeParagraph' or self.setting['whichKind'] == 'articleComment'):
            # 处理的是段落相关的表
            if(type(self.setting['tableName2Clear'])==str):
                # 传入的等待清空的是表名字符串
                # 2 清空数据库表的sql
                sql4truncate = "TRUNCATE `{}`.`{}`;".format(
                    self.setting["databaseName"],
                    self.setting['tableName2Clear']
                )
                dbOperator = dbOp.dbOperator(self.setting["databaseName"])
                # 清空表之前先复制的操作已经放在了上传数据完成的后面
                # dbOperator.cursor.execute(sql4copy2tb_posted)
                dbOperator.cursor.execute(sql4truncate)
                dbOperator.closeDb()
            else:
                # 传入的等待清空的是表名列表
                for table in self.setting['tableName2Clear']:
                    # 2 清空数据库表的sql
                    sql4truncate = "TRUNCATE `{}`.`{}`;".format(
                        self.setting["databaseName"],
                        table
                    )

                    dbOperator = dbOp.dbOperator(self.setting["databaseName"])
                    # 清空表之前先复制的操作已经放在了上传数据完成的后面
                    # dbOperator.cursor.execute(sql4copy2tb_posted)
                    dbOperator.cursor.execute(sql4truncate)
                dbOperator.closeDb()

        elif(self.setting['whichKind'] == 'thumbnailImgs'):
            if (type(self.setting['tableName2Clear']) == str):
                # 传入的等待清空的是表名字符串
                # 1 清空数据库表的sql
                sql4truncate = "TRUNCATE `{}`.`{}`;".format(
                    self.setting["databaseName"],
                    self.setting['tableName2Clear']
                )
                dbOperator = dbOp.dbOperator(self.setting["databaseName"])
                dbOperator.cursor.execute(sql4truncate)
                dbOperator.closeDb()
            else:
                # 传入的等待清空的是表名列表
                # 处理的是缩略图相关的表
                for table in self.setting['tableName2Clear']:
                    # 2 清空数据库表的sql
                    sql4truncate = "TRUNCATE `{}`.`{}`;".format(
                        self.setting["databaseName"],
                        table
                    )
                    # 清空表之前先复制
                    dbOperator = dbOp.dbOperator(self.setting["databaseName"])
                    dbOperator.cursor.execute(sql4truncate)
                dbOperator.closeDb()

        elif (self.setting['whichKind'] == 'contentImgs'):

            if (type(self.setting['tableName2Clear']) == str):
                # 传入的等待清空的是表名字符串
                # 2 清空数据库表的sql
                sql4truncate = "TRUNCATE `{}`.`{}`;".format(
                    self.setting["databaseName"],
                    self.setting['tableName2Clear']
                )
                # 清空表之前先复制
                dbOperator = dbOp.dbOperator(self.setting["databaseName"])
                dbOperator.cursor.execute(sql4truncate)
                dbOperator.closeDb()
            else:
                # 传入的等待清空的是表名列表
                # 处理的是缩略图相关的表
                for table in self.setting['tableName2Clear']:
                    # 2 清空数据库表的sql
                    sql4truncate = "TRUNCATE `{}`.`{}`;".format(
                        self.setting["databaseName"],
                        table
                    )
                    # 清空表之前先复制
                    dbOperator = dbOp.dbOperator(self.setting["databaseName"])
                    dbOperator.cursor.execute(sql4truncate)
                dbOperator.closeDb()

        elif(self.setting['whichKind'] == 'video'):
            if (type(self.setting['tableName2Clear']) == str):
                # 传入的等待清空的是表名字符串
                # 2 清空数据库表的sql
                sql4truncate = "TRUNCATE `{}`.`{}`;".format(
                    self.setting["databaseName"],
                    self.setting['tableName2Clear']
                )
                # 清空表之前先复制
                dbOperator = dbOp.dbOperator(self.setting["databaseName"])
                dbOperator.cursor.execute(sql4truncate)
                dbOperator.closeDb()
            else:
                # 传入的等待清空的是表名列表
                # 处理视频相关的表
                for table in self.setting['tableName2Clear']:
                    # 2 清空数据库表的sql
                    sql4truncate = "TRUNCATE `{}`.`{}`;".format(
                        self.setting["databaseName"],
                        table
                    )
                    # 清空表之前先复制
                    dbOperator = dbOp.dbOperator(self.setting["databaseName"])
                    dbOperator.cursor.execute(sql4truncate)
                dbOperator.closeDb()




import requests

"""
    下载相关的基类和方法
"""

# 下载类
class Downloader:
    # 已有图片链接下载图片的方法
    @staticmethod
    def download_img(urlpath, imgname, dstDirPath):
        '''
        图片下载
        :param urlpath: 可下载的图片资源链接
        :param imgname: 保存图片为指定名字
        :param dstDirPath: 图片保存目录位置，注意：最后要有\\
        :return:
        '''
        r = requests.get(urlpath)
        img = r.content  # 响应的二进制文件
        with open(dstDirPath + str(imgname) + '.png', 'wb') as f:  # 二进制写入
            f.write(img)

    # 下载视频
    @staticmethod
    def downVideo(urlpath, name, dstDirPath, headers_):
        # 获取当前日期
        r = requests.get(urlpath, verify=False, headers=headers_, timeout=30)
        video = r.content  # 响应的二进制文件
        with open(dstDirPath + str(name) + '.mp4', 'wb') as f:  # 二进制写入
            f.write(video)
        r.close()  # 关闭很重要，确保不要过多的连接




import os, hashlib

import cv2
import numpy as np

'''
    针对图片的基础操作
'''
class Contraler_Img:
    @staticmethod
    def get_imgHeight(img_path):
        """获取图片的高度"""
        img = cv2.imdecode(np.fromfile(img_path, dtype=np.uint8), -1)
        h = img.shape[0]
        return h

    @staticmethod
    def cutting_single_img(img_path, cutBottomHeight):
        """裁切单张图片 并且覆盖原图片"""
        img = cv2.imdecode(np.fromfile(img_path, dtype=np.uint8), -1)
        if (img is not None):
            h = img.shape[0]
            w = img.shape[1]
            cutHeight = h - cutBottomHeight
            cutWidth = w
            print('img_shape: ', img.shape, 'cutWidth:cutHeight ', cutWidth, ":", cutHeight)
            cropped = img[0:cutHeight, 0:cutWidth]  # 裁剪坐标为[y0:y1, x0:x1]
            try:
                # cv2.imwrite(imgSrc, cropped) #这种方式保存命名中文会乱码
                cv2.imencode('.jpg', cropped)[1].tofile(img_path)
            except:
                print("图片裁切失败： ", img_path)

    @staticmethod
    def cutting_imgs_dir(dirpath, cutBottomHeight=55):
        '''
            以指定裁切高度裁切指定目录下的所有图片
        :param dirpath 待处理的目录  路径结尾需要\\结尾
        :cutBottomHeight 指定裁切高度
        '''
        imgName_list = os.listdir(dirpath)
        for imgName in imgName_list:
            imgpath = dirpath + imgName
            Contraler_Img.cutting_single_img(img_path=imgpath, cutBottomHeight=cutBottomHeight)

    @staticmethod
    def change_imgMD5(imgSrc):
        """修改单张图片的md5"""
        with open(imgSrc, 'rb') as f:
            md5 = hashlib.md5(f.read()).hexdigest()
        file = open(imgSrc, 'rb').read()
        with open(imgSrc, 'wb') as new_file:
            new_file.write(file + bytes('\0', encoding='utf-8'))  # here we are adding a null to change the file content
            newMD5 = hashlib.md5(open(imgSrc, 'rb').read()).hexdigest()
        print("修改MD5的文件：", imgSrc,"\n旧MD5: ", md5, " \t 新MD5： ",newMD5)

    @classmethod
    def change_imgMD5_dir(cls, dirPath):
        """批量修改目录下图片的MD5"""
        imgNameList = os.listdir(dirPath)
        for imgName in imgNameList:
            imgSrc = dirPath + '\\' + imgName
            cls.changeMD5(imgSrc)






from .globalTools import globalTools
from .videoRef.universalTools import tools
from .videoRef.Spider import bySelenium



# Selenium 爬取 爬取视频源 抖音
def run_douyin(proj_absPath, crawlUrl_list, oriDomain='douyin'):
    updateTime = tools.getCurDate()
    videoDirPath = proj_absPath + '\\assets\\videos\\' + updateTime + '\\' + oriDomain + '\\'
    coverSavedPath = proj_absPath + '\\assets\\videos\\' + updateTime + '\\cover_douyin.jpg'
    captchaPath = proj_absPath + '\\assets\\captcha\\' + updateTime + '\\' + oriDomain + '\\'
    # 判断配置里的目录是否存在，不存在则创建对应目录
    tools.checkACreateDir(videoDirPath)
    tools.checkACreateDir(captchaPath)

    # 抖音视频的爬取及上传
    spider_douyin = bySelenium.crawler_Douyin(captchaPath=captchaPath, videoDirPath=videoDirPath)
    for url in crawlUrl_list:
        lis = spider_douyin.enterIndexDouyin(move2BottomTimes=1, douyinUrlIndex=url)
        print("可上传的视频数量：", len(lis))
        print("可上传的视频：", lis)
        spider_douyin.getRealVideo(lis, videoDirPath, coverSavedPath)
    spider_douyin.browser0.close()
    spider_douyin.browser1.close()
    globalTools.finishTask()




# 清洗标题内容 如抖音包含标签的删除标签， 若删完标签后标题为空，则不要对应视频
class Cleaner_Title:
    douyin_tags = [
        '#股票', '#股市投资', '#财经', '#金融', '#基金', '#干货', '#财经知识', '#金融',
        '#财经', '#理财', '#股民','#投资', '#干货分享', '#财经知识', '#创作灵感', '#换手率',
        '#股票', '#主力', '#金融', '#量化', '#量化交易', '#股市', '#上证指数','#投资理财',
        '#芬钛计划', '#知识点总结', '@DOU+小助手', '#期货', '#韭菜', '#中银绒业', '#A股', '#收盘',
        '#油价', '#一分钟读懂财经', '#配资', '#超短线', '#情绪接力', '#我要上头条', '#商业思维', '#股民',
        '#短线',
    ]
    @classmethod
    def clean_douyin(cls, title):
        '''
        清洗抖音的标题
        包含标签的删除标签， 若删完标签后标题为空，则不要对应视频
        '''
        title = title.replace(' ', '')
        for tag in cls.douyin_tags:
            if(tag in title):
                title = title.replace(tag, '')
            else:
                continue
        return title

    @classmethod
    def clean_douyin_method2(cls, title):
        if('#' in title):
            title = title[:title.index('#')]
        return title




import re

"""
    通用的一些处理
"""


# 字符串正则处理类
class Handler_String_ByRe:
    def extract_StrByRe(self, string, patternStr=r'[[](.*?)[]]'):
        '''
        正则匹配字符串获取指定内容(匹配[]包括这的内容)
        :param string: 待匹配的字符串
        :param patternStr: 正则表达式模式
        :return: 列表
        '''
        pattern = re.compile(patternStr, re.S)  # 最小匹配
        return re.findall(pattern, string)



class Handle_PackageInfo:
    '''
        处理报文信息 如 cookie和headers字符串和对象的转换
    '''
    def translate_Cookies_Row2Obj(self, cookiesRow):
        cookieList = cookiesRow.split(";")
        self.cookies = {}
        for cookieItem in cookieList:
            i = cookieItem.strip().split("=")
            k = i[0]
            v = i[1]
            self.cookies[k] = v
        return self.cookies

    def translate_Headers_Row2Obj(self, headersRow):
        headerList = headersRow.split('\n')
        self.headers = {}
        for headerItem in headerList:
            i = headerItem.strip().split(":")
            if (i != ['']):
                k = i[0]
                v = i[1]
                self.headers[k] = v
        return self.headers


from . import base_poster
import requests
from requests_toolbelt import MultipartEncoder

class Poster_Comment(base_poster.Base_Poster):
    def __init__(self, interface):
        self.interface = interface

    def update_postedurldb(self, item):
        sql = "INSERT INTO `postedurldatabase`.`tb_comment_posted` (`comment`) VALUES (\'{}\');".format(item[0].strip())
        return self.dbOperator.insertData2DB(sql)




'''
    底层功能设计
'''
__all__ = [
    'ContralerDatabase', 'ContralerDir', 'ContralerTime', 'ContralSelenium', 'Download',
    'Encode', 'Handler', 'IsCheck', 'Timer', 'Tools'
]
# - 定时任务器Timer
'''
    - 爬虫定时器参数配置
        setting4Spider_bilibili = {
            "spiderPath": "E:\\Projects\\Crawl_Dealwith_Post_Auto\\video_bilibili_Crawl_Dealwith_Post_Auto\\video_bilibili_Crawl_Dealwith_Post_Auto",
            "spiderName": "bilibiliSpider",
            "command": "Scrapy crawl bilibiliSpider",
            "beginTime" : '08:00:00', # 注意表示 一位数字的要0开头
            "endTime" : '17:50:00',
            "taskExcuteDelta" : 3600,  # 任务间隔1h
            "timeExcuteDelta" : 43200,   # 定时器间隔 每个一天运行一次
        }
        
    - 数据处理及上传定时器参数配置
        setting4AutoDP_bilibili = {
            "beginTime" : '08:00:00', # 注意表示 一位数字的要0开头
            "endTime" : '17:50:00',
            "taskExcuteDelta" : 3600,  # 任务间隔1h
            "timeExcuteDelta" : 43200,   # 定时器间隔 每个一周运行一次
    
            "whichKind": 'video',
            'crawlMethod': 'Scrapy',
            'databaseName': 'videodatabase',
            'databaseUser': 'root',
            'databasePasswd': 'root',
            'tableName': 'tb_bilibili_videoinfo',
            'tableName2Clear': 'tb_bilibili_videoinfo',
            'videoDirPath': 'E:\\test\\',
            'coverSavedPath': 'E:\\test\\cur_Cover.jpg'
        }
        
    - 数据库对应上传过的数据清除的类(没用上)
        setting = {
            "beginTime": '08:00:00',  # 注意表示 一位数字的要0开头
            "endTime": '09:00:00',
            "excuteDelta": 3600,  # 间隔1h 也就是说一天执行一次
            "whichKind": 'keyParagraph',
            "databaseName" : ['stocksnamecode'],   # 对应的数据库名列表
            "tableName" : ['tb_namecode']    # 待清空的表名
        }
            
'''


'''
    定时任务仓库 存放一些定时任务的类和方法
'''
__all__ = [
    'task_video_auto', 'task_comment_auto', 'task_selenium_auto', 'task_contentImgs_auto', 'task_thumbnailImgs_auto',
    'task_keyParagraph_auto', 'task_relativeParagraph_auto'
]
# 主要任务
'''
    1 段落内容的处理及上传
    2 图片内容的下载处理及上传
    3 评论内容的处理及上传
    4 视频内容的下载处理及上传
    （处理包括筛选过滤和清除）
    
    配置文件：
        
'''



from basement__.ContralSelenium import Base_Selenium

from customFunction__.Poster import poster_article
import time

class Crawler_toutiao(Base_Selenium):
    def click_chaijing(self, browser):
        caijing = browser.find_elements_by_xpath('//ul[@class="feed-default-nav"]//li[@aria-label="财经频道"]//div[@class="show-monitor"]')[1]
        self.click_element(caijing)

    def get_articleUrllist(self, browser):
        liList = browser.find_elements_by_xpath("//h2")
        urlList = []
        for li in liList:
            url = li.find_element_by_xpath(".//..").get_attribute('href') # 父节点
            try:
                publishTime = li.find_element_by_xpath(".//..//..").find_element_by_xpath(".//div[@class='feed-card-footer-time-cmp']").text
            except Exception as e:
                continue
            if(not self.check_if_todaypub(publishTime)):
                continue
            if(url not in urlList):
                urlList.append(url)
        return urlList

    def check_if_todaypub(self, s:str):
        """验证是否当天"""
        if('小时' in s):
            return True
        else:
            return False

    def filter_paragraph(self, s):
        word_lis = [
            '新闻记者 ', '责任编辑：', '图片编辑：', '校对：', '内容仅供参考', '编辑 ', '校对 ', '财经记者 ', '请私信', '编辑/', '作者：', '本文源自', '来源：',
            '版权声明', '原创'
        ]
        check = False
        for word in word_lis:
            if(word in s):
                check = True
                return check
        return check

    def get_Content(self, browser, url):
        """获取文章内容 包括里面的标签 之后传送前再根据需求清洗 清洗的步骤放在了这里"""
        content = ''
        browser.get(url)
        title = browser.find_element_by_xpath('//h1').text
        # content = browser.find_element_by_xpath('//article').get_property("innerHTML")
        content_item_lis = browser.find_elements_by_xpath('//article/*')
        if(browser.find_elements_by_xpath("//article//table")!=[]):
            # 有表格的不要
            return (url, title, '')

        for item in content_item_lis:
            if(item.tag_name == 'p'):
                if(item.text!=''):
                    if(self.filter_paragraph(item.text)):
                        continue
                    content  = content + '<p>' + item.text + '</p>'
            else:
                if('<p' in item.get_property("innerHTML") and '<img' in item.get_property("innerHTML")):
                    plis = item.find_elements_by_xpath(".//p")
                    s = ''
                    for pli in plis:
                        s = s + pli.text
                    content = content + '<img src=\'' + item.find_element_by_xpath(".//img").get_attribute('src') + '\'></img>'
                    if(s!=''):
                        content = content + '<p>' + s + '</p>'
                elif('<img' in item.get_property("innerHTML")):
                    content  = content  + '<img src=\'' + item.find_element_by_xpath(".//img").get_attribute('src') + '\'></img>'
                elif('<p' in item.get_property("innerHTML")):
                    p_lis = item.find_elements_by_xpath(".//p")
                    for p_i in p_lis:
                        if(p_i.text!=''):
                            content  = content  + '<p>' + p_i.text + '</p>'
        return (url, title, content)

    def get_articleContent(self, browser):
        urlList = self.get_articleUrllist(browser)
        articleList = []
        print('爬取今日头条文章， 符合发布时间条件的文章数量：', len(urlList))
        for url in urlList:
            time.sleep(2)
            try:
                article = self.get_Content(browser, url)
                articleList.append(article)
            except Exception as e:
                print("该链接获取文章内容失败", url)
        return articleList









import requests, hashlib
from ..universalTools import tools
from requests_toolbelt import MultipartEncoder
from ..DatabaserOperator import databaseOperator as dbOp

class Poster():
    def __init__(self, interface):
        self.dbOperator = dbOp.dbOperator(databaseName='postedurldatabase')
        self.interface = interface
        self.userName = 'qin'
        self.password = 'qin123456'
        self.curDate = str(tools.getCurDate())
        self.key = hashlib.md5(('datapool' + self.userName + self.password + self.curDate).encode('utf-8')).hexdigest()

    def update_postedurldb(self, item):
        sql = "INSERT INTO `postedurldatabase`.`tb_comment_posted` (`comment`) VALUES (\'{}\');".format(item[0].strip())
        return self.dbOperator.insertData2DB(sql)


    # 上传单个数据列表的方法
    def poster(self, postableData):
        # 这里传文件的时候用绝对路径传，不然传了之后显示不了
        formData = (postableData)
        m = MultipartEncoder(formData)
        headers2 = {
            "Content-Type": m.content_type
        }
        paragraphPostResult = requests.post(url=self.interface, data=m, headers=headers2)
        # 上传完更新数据库

        return paragraphPostResult


    def post_auto(self, effectiveDataList, whichKind):
        for item in effectiveDataList:
            if(whichKind=='keyParagraph'):
                postableData = {
                    "key": self.key,
                    "account": self.userName,
                    "password": self.password,
                    'content': item[0],
                    'keyword': item[1],
                    'rekeyword': '配资'
                }
                self.poster(postableData=postableData)
            elif(whichKind=='relativeParagraph'):
                postableData = {
                    "key": self.key,
                    "account": self.userName,
                    "password": self.password,
                    'content': item[0],
                    'keyword': item[1]
                }
                self.poster(postableData=postableData)
            elif(whichKind=='articleComment'):
                comment = item[0].strip()
                sql = "SELECT * FROM `postedurldatabase`.`tb_comment_posted` where `comment` = \'{}\';".format(comment)
                if(self.dbOperator.getOneDataFromDB(sql)):
                    # 数据库中存在对应评论
                    continue
                postableData = {
                    "key": self.key,
                    "account": self.userName,
                    "password": self.password,
                    'comment': item[0].replace(item[1], '股票').replace('&nbsp;',' ')
                }
                self.poster(postableData=postableData)
                # 更新数据库
                self.update_postedurldb(item)


from . import base_filter
from ..Cleaner import cleaner_paragraph

class Filter_Relativeparagraph(base_filter.Base_Filter):
    # -> 关联段落的集成操作方法
    def integratedOp(self, paragraphList, keywordList):
        result = []
        cleanerInstance = cleaner_paragraph.Cleaner_Paragraph()
        for item in paragraphList:
            # 1 进行清洗操作
            item[1] = cleanerInstance.integratedOp(item[1])
            for keyword in keywordList:
                # 2 根据字符串长度125-250间进行筛选
                if (not self.filter_BetweenNumberOfWords(item[1], whichKind='relativeParagraph')):
                    # 段落字数再125-250间，有用可传
                    continue
                # 3 对每个段落进行关键词筛选 若有关键词则跳出当前循环
                if (self.checkIfHasKeyword_relativeParagraph(item[1], keyword)):
                    # 4 根据筛选结果导入可上传的数据
                    result.append(
                        (
                            item[1],  # 段落内容
                            keyword,  # 相关关键词
                        )
                    )
                    break
        return result


"""
    定时器相关的基类和方法
"""
import datetime, os, time
import subprocess, threading
import ctypes, inspect
from . import ContralerDatabase as dbOp
from taskLibrary__ import task_comment_auto, task_video_auto, task_selenium_auto, task_contentImgs_auto, task_thumbnailImgs_auto,task_keyParagraph_auto, task_relativeParagraph_auto, task_article_auto

# 定时任务器基类(设计多个基类)
class Base_Timer_0:
    """每隔指定间隔时间，在指定时间段内，指定时间间隔执行一次任务"""
    def __init__(self, timerConfig):
        self.timerConfig = timerConfig
        self.beginTime = self.timerConfig['beginTime']  # 定时任务每天的启动时间 例子早上8点 08:00:00
        self.endTime = self.timerConfig['endTime']  # 定时任务执行在几点之前 例子早上9点 09:00:00
        self.taskExcuteDelta = self.timerConfig['taskExcuteDelta']  # 定时任务间隔几秒执行一次
        self.timeExcuteDelta = self.timerConfig['timeExcuteDelta']

    def timer_entrance(self):
        '''
        定时器入口，设置每隔指定时间间隔，执行一次定时器
        :param
        :return
        '''
        self.timer_interval() # 入口执行一次函数，从此刻开始计时
        t = threading.Timer(self.timeExcuteDelta, self.timer_entrance)
        t.start()


    def timer_interval(self):
        '''
        定时器间隔函数 8点-9点，隔3600秒执行一次任务task
        :return:
        '''
        self.task()
        now_time = datetime.datetime.now()
        today_9 = datetime.datetime.strptime(
            str(datetime.datetime.now().year) + '-' + str(datetime.datetime.now().month) + '-' + str(
                datetime.datetime.now().day) + ' ' + str(self.endTime), '%Y-%m-%d %H:%M:%S')
        # 因为定时任务会延后10秒钟执行，所以设置终止条件时，需要提前10秒钟
        if (now_time <= today_9 - datetime.timedelta(seconds=10)):
            print("当前时间 " + str(now_time) + " ,该时间在任务执行时段内，继续间隔 " + str(self.taskExcuteDelta) + " 执行任务")
            t = threading.Timer(self.taskExcuteDelta, self.timer_interval)
            t.start()
        else:
            print("当前时间 " + str(now_time) + " ,该时间不在任务执行时段内，等待 " + str(self.timeExcuteDelta) + " 后再重新开启执行任务定时器")

    def task(self):
        '''
        定时任务内容，通过继承重写
        :return:
        '''
        print(' ---------- task任务开始执行 ---------- ')
        print("本定时器未设置定时任务内容")
        time.sleep(1)
        print(' ---------- task任务执行完毕 ---------- ')

    def timerTaskRun(self):
        # 获取当前时间
        now_time = datetime.datetime.now()
        # 获取当前年月日
        now_year = now_time.year
        now_month = now_time.month
        now_day = now_time.day

        # 今天早上8点时间表示
        # today_8 = datetime.datetime.strptime(str(now_year)+'-'+str(now_month)+'-'+str(now_day)+' '+'08:00:00','%Y-%m-%d %H:%M:%S')
        today_8 = datetime.datetime.strptime(str(now_year) + '-' + str(now_month) + '-' + str(now_day) + ' ' + str(self.beginTime), '%Y-%m-%d %H:%M:%S')
        # 明天早上8点时间表示
        tomorrow_8 = datetime.datetime.strptime(str(now_year) + '-' + str(now_month) + '-' + str(now_day) + ' ' + str(self.beginTime), '%Y-%m-%d %H:%M:%S')

        # 判断当前时间是否过了今天凌晨8点,如果没过，则今天早上8点开始执行，过了则从明天早上8点开始执行，计算程序等待执行的时间
        if (now_time <= today_8):
            wait_time = (today_8 - now_time).total_seconds()
        else:
            wait_time = (tomorrow_8 - now_time).total_seconds()

        # 等待wait_time秒后（今天早上8点或明天早上8点），开启线程去执行func函数
        self.thread = threading.Timer(wait_time, self.timer_entrance)  # 当前线程
        self.thread.start()


# 结束线程的类没有用到
class StopThread():
    '''结束线程的类'''
    def _async_raise(self,tid, exctype):
        """raises the exception, performs cleanup if needed"""
        tid = ctypes.c_long(tid)
        if(not inspect.isclass(exctype)):
            exctype = type(exctype)
        res = ctypes.pythonapi.PyThreadState_SetAsyncExc(tid, ctypes.py_object(exctype))
        if(res == 0):
            raise ValueError("invalid thread id")
        elif(res != 1):
            # """if it returns a number greater than one, you're in trouble,
            # and you should call it again with exc=NULL to revert the effect"""
            ctypes.pythonapi.PyThreadState_SetAsyncExc(tid, None)
            raise SystemError("PyThreadState_SetAsyncExc failed")

    # 任务自动执行结束 结束本线程实例，在调用本方法之前得先调用 timedTaskRun()
    def stopCurThread(self, thread):
        self._async_raise(thread.ident, SystemExit)


'''
    下面是定时器特定任务定制，继承自基类的定时器
'''

# 1. 定时任务 ———— 爬取数据的的类
class TaskTimer_Spider(Base_Timer_0):
    def task(self):
        self.setting = self.timerConfig
        if(self.setting['tasktype']=='scrapy'):
            # 通过os改变工作路径,注意路径是绝对路径，而且还要是\\  只要环境是同一个 这样可以执行对应的爬虫项目
            os.chdir(self.setting['spiderPath'])
            subprocess.Popen(self.setting['command'])
        elif(self.setting['tasktype']=='selenium'):
            task_selenium_auto.run_spider(self.setting["origin"])
        else:
            print('tasktype参数出错')


# 2. 定时任务 ———— 处理数据及上传 段落数据 到接口的类（完成）
class TaskTimer_AutoDealwithPost(Base_Timer_0):
    def task(self):
        self.setting = self.timerConfig
        if(self.setting["whichKind"] == 'keyParagraph'):
            task_keyParagraph_auto.run(setting=self.setting)
        elif(self.setting["whichKind"] == 'relativeParagraph'):
            task_relativeParagraph_auto.run(setting=self.setting)
        elif(self.setting["whichKind"] == 'contentImgs'):
            task_contentImgs_auto.run(proj_absPath=self.setting["proj_absPath"], oriDomain=self.setting["oriDomain"], database=self.setting['databaseName'], tableNameList=self.setting['tableName'], maskFilt=self.setting['maskFilt'])
        elif(self.setting["whichKind"] == 'thumbnailImgs'):
            task_thumbnailImgs_auto.run(proj_absPath=self.setting["proj_absPath"], oriDomain=self.setting["oriDomain"], database=self.setting['databaseName'], tableNameList=self.setting['tableName'])
        elif (self.setting['whichKind'] == 'video' and self.setting['crawlMethod'] == 'Selenium'):
            task_selenium_auto.run_douyin(proj_absPath=self.setting["proj_absPath"], crawlUrl_list=self.setting['crawlUrl_list'],oriDomain=self.setting["oriDomain"])
        elif(self.setting["whichKind"]=='video' and self.setting['crawlMethod'] == 'Scrapy'):
            task_video_auto.run_bilibili(setting=self.setting)
        elif(self.setting["whichKind"]=='articleComment'):
            task_comment_auto.run(setting=self.setting)
        elif(self.setting['whichKind']=='articles'):
            task_article_auto.run(setting=self.setting)
        print("上传数据完成，接下来清空数据库")
        # 上传完直接清空数据库，不用再用定时器
        clearDB = TimedTask4AutoClearDB(timerConfig=self.setting)
        clearDB.task()


# 3. 定时任务 ———— 数据库对应上传过的数据清除的类
class TimedTask4AutoClearDB(Base_Timer_0):
    def task(self):
        '''
        setting = {
            "beginTime": '08:00:00',  # 注意表示 一位数字的要0开头
            "endTime": '09:00:00',
            "excuteDelta": 3600,  # 间隔1h 也就是说一天执行一次
            "whichKind": 'keyParagraph',
            "databaseName" : ['stocksnamecode'],   # 对应的数据库名列表
            "tableName" : ['tb_namecode']    # 待清空的表名
        }
        '''
        self.setting = self.timerConfig
        if(self.setting['whichKind'] == 'keyParagraph' or self.setting['whichKind'] == 'relativeParagraph' or self.setting['whichKind'] == 'articleComment'):
            # 处理的是段落相关的表
            if(type(self.setting['tableName2Clear'])==str):
                # 传入的等待清空的是表名字符串
                # 2 清空数据库表的sql
                sql4truncate = "TRUNCATE `{}`.`{}`;".format(
                    self.setting["databaseName"],
                    self.setting['tableName2Clear']
                )
                dbOperator = dbOp.Contraler_Database(self.setting["databaseName"])
                # 清空表之前先复制的操作已经放在了上传数据完成的后面
                # dbOperator.cursor.execute(sql4copy2tb_posted)
                dbOperator.cursor.execute(sql4truncate)
                dbOperator.closeDb()
            else:
                # 传入的等待清空的是表名列表
                for table in self.setting['tableName2Clear']:
                    # 2 清空数据库表的sql
                    sql4truncate = "TRUNCATE `{}`.`{}`;".format(
                        self.setting["databaseName"],
                        table
                    )

                    dbOperator = dbOp.Contraler_Database(self.setting["databaseName"])
                    # 清空表之前先复制的操作已经放在了上传数据完成的后面
                    # dbOperator.cursor.execute(sql4copy2tb_posted)
                    dbOperator.cursor.execute(sql4truncate)
                dbOperator.closeDb()

        elif(self.setting['whichKind'] == 'thumbnailImgs'):
            if (type(self.setting['tableName2Clear']) == str):
                # 传入的等待清空的是表名字符串
                # 1 清空数据库表的sql
                sql4truncate = "TRUNCATE `{}`.`{}`;".format(
                    self.setting["databaseName"],
                    self.setting['tableName2Clear']
                )
                dbOperator = dbOp.Contraler_Database(self.setting["databaseName"])
                dbOperator.cursor.execute(sql4truncate)
                dbOperator.closeDb()
            else:
                # 传入的等待清空的是表名列表
                # 处理的是缩略图相关的表
                for table in self.setting['tableName2Clear']:
                    # 2 清空数据库表的sql
                    sql4truncate = "TRUNCATE `{}`.`{}`;".format(
                        self.setting["databaseName"],
                        table
                    )
                    # 清空表之前先复制
                    dbOperator = dbOp.Contraler_Database(self.setting["databaseName"])
                    dbOperator.cursor.execute(sql4truncate)
                dbOperator.closeDb()

        elif (self.setting['whichKind'] == 'contentImgs'):

            if (type(self.setting['tableName2Clear']) == str):
                # 传入的等待清空的是表名字符串
                # 2 清空数据库表的sql
                sql4truncate = "TRUNCATE `{}`.`{}`;".format(
                    self.setting["databaseName"],
                    self.setting['tableName2Clear']
                )
                # 清空表之前先复制
                dbOperator = dbOp.Contraler_Database(self.setting["databaseName"])
                dbOperator.cursor.execute(sql4truncate)
                dbOperator.closeDb()
            else:
                # 传入的等待清空的是表名列表
                # 处理的是缩略图相关的表
                for table in self.setting['tableName2Clear']:
                    # 2 清空数据库表的sql
                    sql4truncate = "TRUNCATE `{}`.`{}`;".format(
                        self.setting["databaseName"],
                        table
                    )
                    # 清空表之前先复制
                    dbOperator = dbOp.Contraler_Database(self.setting["databaseName"])
                    dbOperator.cursor.execute(sql4truncate)
                dbOperator.closeDb()

        elif(self.setting['whichKind'] == 'video'):
            if (type(self.setting['tableName2Clear']) == str):
                # 传入的等待清空的是表名字符串
                # 2 清空数据库表的sql
                sql4truncate = "TRUNCATE `{}`.`{}`;".format(
                    self.setting["databaseName"],
                    self.setting['tableName2Clear']
                )
                # 清空表之前先复制
                dbOperator = dbOp.Contraler_Database(self.setting["databaseName"])
                dbOperator.cursor.execute(sql4truncate)
                dbOperator.closeDb()
            else:
                # 传入的等待清空的是表名列表
                # 处理视频相关的表
                for table in self.setting['tableName2Clear']:
                    # 2 清空数据库表的sql
                    sql4truncate = "TRUNCATE `{}`.`{}`;".format(
                        self.setting["databaseName"],
                        table
                    )
                    # 清空表之前先复制
                    dbOperator = dbOp.Contraler_Database(self.setting["databaseName"])
                    dbOperator.cursor.execute(sql4truncate)
                dbOperator.closeDb()


'''
    自动化引擎
        缩略图 清洗 筛选 并上传
'''
from globalTools import globalTools
from customFunction__.Poster import poster_img
from customFunction__.imgref import classifier as Classifier
from customFunction__.imgref import processing as Processing
from customFunction__.Filter import filter_image
from basement__ import ContralerDatabase as dbOp
from basement__.ContralerTime import Contraler_Time
from basement__.ContralerDir import Contraler_Dir
from basement__.Download import Downloader

def run(proj_absPath, oriDomain, database, tableNameList):
    # 将目录下的图片图片传到接口
    # 获取当前目录所在绝对路径
    # proj_absPath = os.path.abspath(os.path.dirname(__file__))
    updateTime = Contraler_Time.getCurDate("%Y%m%d")
    '''
    oriDomain = 'huxiu'
    '''
    setting = {
        # 爬取下来的图片的存放路径
        'imgsCrawledDir' : proj_absPath + '\\assets\imgsCrawled\\' + updateTime + '\\' + oriDomain + '\\',
        # 经过百度识别重命名后存放的目录路径
        'imgsReconizedDir': proj_absPath + '\\assets\imgsReconized\\' + updateTime + '\\' + oriDomain + '\\',
        # 初步处理过后的无水印的图片的目录
        'imgsDirDontHasWaterMask' : proj_absPath + '\\assets\imgsDontHasWaterMask\\' + updateTime + '\\' + oriDomain + '\\',
        # 初步处理过后有水印的图片的目录
        'imgsDirHasWaterMask' : proj_absPath + '\\assets\imgDirHasWaterMask\\' + updateTime + '\\' + oriDomain + '\\',
        # 处理完成的图片目录
        'imgsCleanedDir' : proj_absPath + '\\assets\imgsCleanedDir\\' + updateTime + '\\' + oriDomain + '\\',
        # 缩略图保存位置图片目录
        'imgsThumbnailDir' : proj_absPath + '\\assets\imgsThumbnailDir\\' + updateTime + '\\' + oriDomain + '\\',
    }

    # 判断配置里的目录是否存在，不存在则创建对应目录
    for item in setting.values():
        Contraler_Dir.checkACreateDir(item)

    # 从数据库获取图片链接 下载图片
    print("从数据库获取图片链接")
    dbOperator = dbOp.Contraler_Database(database)
    # 上传过的图片去重
    picList_posted_ = dbOperator.getAllDataFromDB("SELECT `origin_pic_path` FROM `postedurldatabase`.`tb_thumbnailimgs_posted`")
    picList_posted = []
    for item in picList_posted_:
        picList_posted.append(item[0])
    print(picList_posted)
    print("下面开始下载图片")
    for table in tableNameList:
        sql = "SELECT `id`,`origin_pic_path` FROM `" + table + "`;"
        imgUrlPathList = dbOperator.getAllDataFromDB(sql)
        print("数据表为 ", table, "获得图片链接数量： ", len(imgUrlPathList))
        for imgUrl in imgUrlPathList:
            if(imgUrl[1] in picList_posted):
                continue
            imgName = table + "_" + str(imgUrl[0])
            Downloader.download_img(urlpath=imgUrl[1], imgname=imgName, dstDirPath=setting["imgsCrawledDir"])
            insert_SQL = "INSERT INTO `postedurldatabase`.`tb_thumbnailimgs_posted` (`origin_pic_path`) VALUES (\'{}\');".format(imgUrl[1])
            dbOperator.insertData2DB(insert_SQL)

    print("图片下载完成")
    # 对爬取下来的图片进行处理 - 识别重命名、过滤、水印的识别及裁切 处理完后放在路径  ./assets/imgsCleanedDir 下
    # 2 重命名
    classifier = Classifier.imgsClassifier(crawledDirPath=setting['imgsCrawledDir'], savedDirPath=setting['imgsReconizedDir'])
    classifier.run()

    # 3 过滤 (关键词过滤、空文件过滤、水印识别及处理）
    print("下面开始过滤操作")
    filter = filter_image.imgsFilter(
        imgsDontHasWaterMaskDir=setting['imgsDirDontHasWaterMask'],
        imgDirHasWaterMask=setting['imgsDirHasWaterMask'],
        imgCleanedStep1 = setting['imgsCleanedDir'],
        # dirOriPath=setting['imgsReconizedDir']
        dirOriPath = setting['imgsCrawledDir']
    )
    filter.run_()    # 这里再做一下优化
    print("过滤操作完成")

    # 4 将过滤后的图片整成缩略图
    print("下面将图片整成缩略图")
    thumbnailOriDir = setting['imgsReconizedDir']
    thumbnailDstDir = setting['imgsThumbnailDir']
    imgprocesser = Processing.Imgs2ThumbnailByDir(thumbnailOriDirPath=thumbnailOriDir, thumbnailDstDirPath=thumbnailDstDir)
    imgprocesser.cutOff2ThumbnailByDir()

    print("下面将开始传送")
    # # 4 创建图片发送的poster 传送处理完成的图片
    # 传送缩略图
    imgposter0 = poster_img.Poster_Imgs(imgDirPath=setting['imgsThumbnailDir'])
    imgposter0.updateImgsThumbnail()
    globalTools.finishTask()



'''
    自动化引擎
        从评论数据库中获取数据 进行 清洗 筛选 并上传 （已完成）
'''
from globalTools import globalTools
from basement__ import ContralerDatabase as dbOp
from customFunction__.Filter import filter_comment,filter_posted
from customFunction__.Poster import poster_comment as Poster

def run(setting):
    # setting = {
    #     'databaseName' : 'commentdatabase',
    #     'databaseUser' : 'root',
    #     'databasePasswd' : 'root',
    #     'keywordList' : ['个股', '股市', 'A股', '港股', '新股', '美股', '创业板', '证券股', '股票', '炒股', '散户', '短线', '操盘', '波段']
    # }
    # setting['sql'] = "select id,comment from ``.``;"

    # 1 获取对应数据
    dbOperator = dbOp.Contraler_Database(databaseName=setting['databaseName'], user=setting['databaseUser'], passwd=setting['databasePasswd'])
    dataList = dbOperator.getAllDataFromDB(setting['sql'])

    # 2 对所有段落内容判断，若上传过则删除对应上传过的段落
    filter4postedcheck = filter_posted.Filter_Posted()
    dataList = filter4postedcheck.run(dataOriList=dataList)

    # 3 过滤操作 输入的列表为从数据库中获取的列表（过滤操作包含清洗操作，不用单独进行清洗）
    filterInstance = filter_comment.Filter_Comment()
    postableList = filterInstance.integratedOp(commentList=dataList, keywordList=setting['keywordList'])


    # 4 上传列表
    posterInstance = Poster.Poster_Comment(interface='http://121.40.187.51:8088/api/articlecomment_api')
    posterInstance.post_auto(postableList, whichKind='articleComment')

    # # 5 将上传过的数据放到postedurldatabase中
    postedDBOP = dbOp.Contraler_Database(databaseName='postedurldatabase', user='root', passwd='root')
    for comment in postableList:
        sql = "INSERT INTO `postedurldatabase`.`tb_comment_posted` (`comment`) VALUES (\'{}\');".format(comment[0])
        postedDBOP.insertData2DB(sql)
    globalTools.finishTask()


from . import base_cleaner
from basement__ import ContralerDatabase


class Cleaner_Article(base_cleaner.Base_Cleaner):
    """
        文章内容 清洗需求
            1 只保留p标签和img标签
            2 清掉所有样式属性 img保留src
    """
    def __init__(self):
        self.cleanWordList = [
            '本报记者 ', '友情提示：'
        ]
        self.irrelevantWordList = ['图']
        self.baseClass = base_cleaner.Base_Cleaner  # 父类

    def clean_tag(self, content):
        pass

    def integratedOp(self, paragraph):
        # 清序号
        temp = self.baseClass.delHeaderNum(paragraph, cleanWordList=self.cleanWordList)
        # 清空格
        temp = self.baseClass.delSpace(temp)
        # 清序号
        temp = self.baseClass.delHeaderNum(temp, cleanWordList=self.cleanWordList)
        # 清股票
        # temp = self.delStocksNameAndCode(temp)
        # 清不相关的文字
        result = self.baseClass.delIrrelevantWord(temp, irrelevantWordList=self.irrelevantWordList)
        return result




'''
    进行数据库的操作
'''
import pymysql

class dbOperator():
    def __init__(self, databaseName, user='root', passwd='root'):
        self.databaseName = databaseName
        self.user = user
        self.passwd = passwd
        self.openDB()
        pass

    def openDB(self):
        self.conn = pymysql.connect(
            host='localhost',
            user=self.user,
            passwd=self.passwd,
            db=self.databaseName,
            autocommit=True
        )
        self.cursor = self.conn.cursor()
        pass

    def getAllDataFromDB(self, sql):
        try:
            self.cursor.execute(sql)
            data = self.cursor.fetchall()
            # 将数据转换成列表
            result = []
            for item in data:
                result.append(
                    [
                        item[i] for i in range(len(item))
                    ]
                )
            return result
        except Exception as e:
            print(e)
            return -1

    def getOneDataFromDB(self, sql):
        try:
            self.cursor.execute(sql)
            data = self.cursor.fetchone()
            return data
        except Exception as e:
            print(e)
            return -1

    def insertData2DB(self, sql):
        try:
            self.cursor.execute(sql)
        except Exception as e:
            print(e)
            return -1

    def closeDb(self):
        self.cursor.close()
        self.conn.close()
        pass



from basement__.ContralSelenium import Base_Selenium
from basement__.ContralerDatabase import Contraler_Database
def start_chrome(chromeDriverPath):
    c = Base_Selenium()
    driver = c.init_webdriver(chromeDriverPath=chromeDriverPath)
    executor = driver.command_executor._url
    session_id = driver.session_id
    print("driver executor : ", executor)
    print("driver session id: ", session_id)
    return {
        "executor": executor,
        "session_id": session_id
    }
c = Base_Selenium()
driver = c.init_webdriver(chromeDriverPath=r'E:\Projects\webDriver\chrome\chromedriver.exe')
dbOperator = Contraler_Database(databaseName='data_usable_database')
executor = driver.command_executor._url
session_id = driver.session_id
sql_update = 'UPDATE `data_usable_database`.`tb_selenium_info` SET `executor` = \'{}\', `session_id` = \'{}\' WHERE (`id` = \'1\');'.format(
    executor,
    session_id
)
dbOperator.cursor.execute(sql_update)
print("driver executor : ", executor)
print("driver session id: ", session_id)




'''
    针对basement的测试
'''
from basement__ import ContralerDatabase, Timer
import os



class Test_ContralerDatabase:
    def __init__(self, databaseName, user, passwd):
        self.contraler_db = ContralerDatabase.Contraler_Database(databaseName, user, passwd)

    def test_openDB(self):
        return self.contraler_db.openDB()

    def test_getAllDataFromDB(self, sql):
        return self.contraler_db.getAllDataFromDB(sql)

    def test_getOneDataFromDB(self, sql):
        return self.contraler_db.getOneDataFromDB(sql)

    def test_insertData2DB(self, sql):
        return self.contraler_db.insertData2DB(sql)

    def test_closeDb(self):
        return self.contraler_db.closeDb()

    def run(self):
        '''
            开始测试
        '''
        print("开始")
        print('1.1 ', self.test_openDB())
        sql_0 = 'select * from tb_comment_gelonghui_content'
        print('1.2 执行sql语句：', sql_0, '\n', self.test_getAllDataFromDB(sql=sql_0))
        sql_1 = 'select * from tb_comment_gelonghui_content where `id`=1'
        print('1.3 执行sql语句：', sql_1, '\n', self.test_getOneDataFromDB(sql=sql_1))
        sql_2 = "INSERT INTO `commentdatabase`.`tb_comment_gelonghui_content` (`comment`, `publishTime`) VALUES ('2', '2');"
        print('1.4 执行sql语句：', sql_2, '\n', self.test_insertData2DB(sql=sql_2))
        print('1.5', self.test_closeDb())
        print("结束")

class Test_Timer_Base_Timer_0:
    def __init__(self):
        self.timerConfig = {
            "beginTime" : '09:06:00', # 注意表示 一位数字的要0开头
            "endTime" : '23:00:00',
            "taskExcuteDelta" : 3600,  # 任务间隔1h
            "timeExcuteDelta" : 86400,   # 定时器间隔 每个一天运行一次
        }
        self.base_timer_0 = Timer.Base_Timer_0(timerConfig=self.timerConfig)

    def test_timerTaskRun(self):
        self.base_timer_0.timerTaskRun()

class Test_TaskTimer_Spider:
    timerConfig = {
        "beginTime": '08:30:00',  # 注意表示 一位数字的要0开头
        "endTime": '23:00:00',
        "taskExcuteDelta": 3600,  # 任务间隔1h
        "timeExcuteDelta": 43200,  # 定时器间隔 每个一天运行一次
        "tasktype": 'selenium',
        "origin": 'kuaishou'
    }
    tasktimer_spider = Timer.TaskTimer_Spider(timerConfig=timerConfig)

    def test_timerTaskRun(self):
        self.tasktimer_spider.timerTaskRun()

class Test_TaskTimer_AutoDealwithPost:
    # 获取当前目录所在绝对路径
    proj_absPath = os.path.abspath(os.path.dirname(__file__))
    oriDomain = 'huxiu'
    timerConfig = {
        "beginTime": '08:32:00',  # 注意表示 一位数字的要0开头
        "endTime": '23:00:00',
        "taskExcuteDelta": 3600,  # 任务间隔1h
        "timeExcuteDelta": 43200,  # 定时器间隔 每个一天运行一次
        "whichKind": 'articleComment',
        'keywordList' : ['个股', '股市', 'A股', '港股', '新股', '炒股', '散户', '短线', '操盘', '波段', '股票'],
        'databaseUser' : 'root',
        'databasePasswd' : 'root',
        "databaseName": 'commentdatabase',  # 对应的数据库名
        "tableName": ['tb_comment_taoguba1_content'],  # 待处理的表名——存放图片url信息的表
        "tableName2Clear": ['tb_comment_taoguba1_content'],
        "sql": 'SELECT id, comment FROM `commentdatabase`.`tb_comment_taoguba1_content`;'
    }
    tasktimer_autodealwithpost = Timer.TaskTimer_AutoDealwithPost(timerConfig=timerConfig)

    def test_timerTaskRun(self):
        self.tasktimer_autodealwithpost.timerTaskRun()


if(__name__=='__main__'):
    print("---------------测试basement__的功能-------------")
    # print("1 ContralerDatabase")
    # test_cdb = Test_ContralerDatabase('commentdatabase', 'root', 'root')
    # test_cdb.run()
    # print("2 Timer_Base_Timer_0")
    # test_tbt = Test_Timer_Base_Timer_0()
    # test_tbt.test_timerTaskRun()
    print("3 TaskTimer_Spider")
    test_tbs = Test_TaskTimer_Spider()
    test_tbs.test_timerTaskRun()
    # print("4 TaskTimer_AutoDealwithPost")
    # test_adp = Test_TaskTimer_AutoDealwithPost()
    # test_adp.test_timerTaskRun()




#-*-coding:utf-8-*-
import os, re
import cv2
import numpy as np
from ..universalTools import tools


'''
    dirPath : 存放待处理的图片的目录路径
        关键词过滤
        水印过滤
        水印处理-图片裁切
'''
class imgsFilter():
    def __init__(self, imgsDontHasWaterMaskDir, imgDirHasWaterMask, imgCleanedStep1, dirOriPath="E:\Projects\imgReconization\imgs\\"):
        self.imgDirPath = dirOriPath  # 存放图片的目录路径
        self.imgNameList = os.listdir(dirOriPath) # 获取目录下所有图片的名字
        self.containFilterStrImgPathList = []   # 含有表格俩字的图片路径的列表
        self.cleanedImgsList = os.listdir(dirOriPath)   # 经过清洗后的图片的路径列表
        self.cleanedImgsListIsChanged = False   # 判断是否经过过滤
        self.imgNameListHasWaterMask = []   # 含有水印的图片列表
        self.imgNameListHasNoWaterMask = [] # 不含有水印的图片列表
        self.containFilterStrImgPathList = []
        self.imgCleanedStep1 = imgCleanedStep1     # 用于保存经过初步筛选（处理水印之前）的图片的目录
        self.imgDirHasWaterMask = imgDirHasWaterMask    # 用于保存含有水印的图片的目录
        self.imgDirDontHasWaterMask = imgsDontHasWaterMaskDir


    # --------------------- 1 图片过滤 依据命名关键词过滤图片 ---------------------------------------------
    # 过滤掉含指定字符 filterStr 的的不通用图片的路径 默认过滤掉表格
    def filterStr(self, filterStr = '表格'):
        self.cleanedImgsListIsChanged = True
        for imgName in self.cleanedImgsList:
            # 过滤掉命名全部为数字，没有关键字的图片
            name = imgName.split(".")[0]
            if(tools.checkIfAllNumber(name)):
                # 全为数字则过滤
                if (imgName not in self.containFilterStrImgPathList):
                    self.containFilterStrImgPathList.append(imgName)
            else:
                if(re.search(filterStr, imgName)):
                    if(imgName not in self.containFilterStrImgPathList):
                        self.containFilterStrImgPathList.append(imgName)


    # 依据关键词过滤掉不能用的图片
    def filterStrList(self, filterStrList = [
        '图表','表格', '截图', '走势图', '书本','数学试卷','说明书','文字图片','图画', '彩色动漫', '便签纸', '评定表', '食谱','电视背景墙','竞买申请书','抛光砖','进口货物报关单','小窝','报纸杂志',
        '手抄报','公共标示','行政区划图','电路图','专业招生简章','成绩单','报告单','派车单','网页导航','简历模板','电脑刷','名片','中国建设银行',
        '中国银行','医药标志','卡巴斯基软件','数学图形','投资银行面试指南','通知书','二维码','简笔画','家用电器','亲吻','表彰会','简报','英语字母','圆珠笔',
        '图标','毛笔','车标','书籍封面','头发','计算器','会刊广告','升国旗','地图','彩票','卡通动漫人物','信用卡','旗帜','平板手机','pvc卡','广告',
        '网址','曲谱','东芝','玩具','电纸书','放大镜','头饰','工笔画','交通路牌','交通标志','t恤','手提包','瓶子','戒指', '显示器屏幕'
    ]):
        self.cleanedImgsListIsChanged = True
        for filterStr in filterStrList:
            self.filterStr(filterStr)
        self.setCleanedImgsList()

    # --------------------- 2 图片过滤 依据图片的尺寸过滤图片 ---------------------------------------------
    # 基础过滤
    def filterNoneFile(self):
        for imgName in self.cleanedImgsList:
            imgSrc = self.imgDirPath + imgName
            img = cv2.imdecode(np.fromfile(imgSrc, dtype=np.uint8), -1)
            if(img is None):
                # 过滤掉打不开的图片
                self.cleanedImgsList.pop(self.cleanedImgsList.index(imgName))
            else:
                if(img.shape[0]<300):
                    # 过滤掉高度很小的图片
                    self.cleanedImgsList.pop(self.cleanedImgsList.index(imgName))

    def filterImgBySizeSingle(self, imgName, imgSrc):
        # 打开图片
        img = cv2.imdecode(np.fromfile(imgSrc, dtype=np.uint8), -1)
        if(img is not None):
            # 获取图片宽高
            height = img.shape[0]
            width = img.shape[1]
            # 判断图片宽高是否满足要求
            if (height < 300):
                return {
                    "check": True,
                    "imgSrc": imgSrc
                }
            else:
                return {
                    "check": False,
                    "imgSrc": imgSrc
                }
        else:
            # 若图片读取不了也过滤掉
            self.cleanedImgsList.pop(self.cleanedImgsList.index(imgName))
            return -1


    def filterImgBySizeAll(self):
        for imgName in self.cleanedImgsList:
            imgSrc = self.imgDirPath + imgName
            result = self.filterImgBySizeSingle(imgName,imgSrc)
            if(result!=-1 and result["check"]):
                # result为true则该图片过滤掉
                index = self.cleanedImgsList.index(imgName)
                self.cleanedImgsList.pop(index)
            else:
                # result为false则该图片保留
                pass

    # --------------------- 3 水印处理 ---------------------------------------------
    '''
        通过识别水印的方式貌似不是那么好，直接采用裁切的形式把水印去掉吧，当然这个方法也不是完美，当至少不会那么麻烦
        判断单张图片是否含有水印 采用cv2的模板匹配方法
    '''
    def checkIfHasWatermarkSingle(self, imgName, imgDirPath="E:\Projects\imgReconization\imgs\\", templateSrc="E:\Projects\imgReconization\dontDelete\\templatePerfectOne28.png"):
        # 获取水印目录下所有水印路径
        templateNameList = os.listdir('E:\Projects\imgReconization\dontDelete\\')
        templatePathList = []
        for item in templateNameList:
            templatePathList.append('E:\Projects\imgReconization\dontDelete\\' + item)

        # img_rgb = cv2.imread(imgSrc)  # 路径含有中文名不能直接用这个，要用下面那个转换
        imgSrc = imgDirPath + imgName
        img_rgb = cv2.imdecode(np.fromfile(imgSrc, dtype=np.uint8), -1)  # Read the main image
        img_gray = cv2.cvtColor(img_rgb, cv2.COLOR_BGR2GRAY)    # Convert it to grayscale



        def cvReconize(img_gray, template):
            res = cv2.matchTemplate(img_gray, template, cv2.TM_CCOEFF_NORMED)  # Perform match operations.
            return res


        threshold = 0.38 # Specify a threshold 0.35本来挺好的
        for templateSrc in templatePathList:
            # 根据图片的尺寸来选择合适的模板

            template = cv2.imread(templateSrc, 0)  # Read the template
            w, h = template.shape[::-1]  # Store width and heigth of template in w and h

            res = cvReconize(img_gray, template)
            loc = np.where(res >= threshold)    # Store the coordinates of matched area in a numpy array
            x = loc[0]
            y = loc[1]
            # Draw a rectangle around the matched region.
            if(len(x) and len(y)):
                for pt in zip(*loc[::-1]):
                    # pt[0]表示水印位置所在的像素高度
                    cv2.rectangle(img_rgb, pt, (pt[0] + w, pt[1] + h), (0, 255, 255), 2)
                    # Show the final image with the matched area.
                    cv2.imwrite("E:\Projects\imgReconization\\reconizeWaterMaskResult\\test_001.png", img_rgb)
                    waterMaskLocY = pt[1]
                print("I found the watermark: ", imgSrc, "水印位置： x= ",x, " y= ",y, "对应模板：", templateSrc)
                return {
                    "hasWaterMask" : True,
                    "imgName" : imgName,
                    "waterMaskLocY": waterMaskLocY      # 水印位置的y坐标
                }
            else:
                print('there is no watermark', imgSrc)
                # return {
                #     "hasWaterMask" : False,
                #     "imgName" : imgName
                # }
                print({
                    "hasWaterMask" : False,
                    "imgName" : imgName
                })
                continue
        print("没有对应的水印模板或无水印")
        return {
            "hasWaterMask" : False,
            "imgName" : imgName
        }

    # 将经过筛选的所有图片依据是否有水印进行分类
    def classifyImgsListByWaterMask(self):
        for imgName in self.cleanedImgsList:
            try:
                result = self.checkIfHasWatermarkSingle(imgName, self.imgDirPath)
                if(result["hasWaterMask"]):
                    self.imgNameListHasWaterMask.append(result["imgName"])
                    src = self.imgDirPath + imgName
                    dst = self.imgDirHasWaterMask + imgName
                    tools.copyFile(src, dst)
                    # 对水印进行裁切
                    waterMaskY = result["waterMaskLocY"]
                    imgHeight = self.get_imgHeight(src)
                    cutBottomHeight = imgHeight - waterMaskY
                    self.cutOffWaterMaskSingle(imgSrc=dst, cutBottomHeight=cutBottomHeight)
                else:
                    self.imgNameListHasNoWaterMask.append(result["imgName"])
                    src = self.imgDirPath + imgName
                    dst = self.imgDirDontHasWaterMask + imgName
                    tools.copyFile(src, dst)
            except Exception as E:
                print(E)

    # --------------------- 4 图片裁切 ---------------------------------------------
    # 获取图片的高度
    def get_imgHeight(self, imgSrc):
        img = cv2.imdecode(np.fromfile(imgSrc, dtype=np.uint8), -1)
        h = img.shape[0]
        return h

    # 单张图片的裁切 用cv2
    def cutOffWaterMaskSingle(self, imgSrc, cutBottomHeight=55):
        # img = cv2.imread(imgSrc)
        img = cv2.imdecode(np.fromfile(imgSrc, dtype=np.uint8), -1)
        if(img is not None):
            try:
                h = img.shape[0]
                w = img.shape[1]
                cutHeight = h - cutBottomHeight
                cutWidth = w
                print(img.shape)
                print(cutWidth, ":", cutHeight)
                cropped = img[0:cutHeight, 0:cutWidth]  # 裁剪坐标为[y0:y1, x0:x1]
                # cv2.imwrite(imgSrc, cropped) #这种方式保存命名中文会乱码
                cv2.imencode('.jpg', cropped)[1].tofile(imgSrc)
            except:
                print(cropped)
                print(imgSrc)

    # 对目录下的所有文件进行裁切并覆盖
    def cutOffWaterMaskByDir(self, cutBottomHeight=55):
        for imgName in self.cleanedImgsList:
            imgSrc = self.imgCleanedStep1 + imgName
            self.cutOffWaterMaskSingle(imgSrc, cutBottomHeight)


    # --------------------- 5 以下为set操作 ---------------------------------------------
    # 设置经过清洗的图片路径列表 本方法用于关键词过滤 见1
    def setCleanedImgsList(self):
        self.cleanedImgsListIsChanged = True
        for imgItem in self.containFilterStrImgPathList:
            delIndex = self.cleanedImgsList.index(imgItem)
            self.cleanedImgsList.pop(delIndex)

    # 将经过初步过滤的图片复制到新的目录下
    def setCopyCleanedImgsList(self):
        for imgName in self.cleanedImgsList:
            src = self.imgDirPath + imgName
            dst = self.imgCleanedStep1 + imgName
            tools.copyFile(src, dst)

    # --------------------- 6 以下为get操作 ---------------------------------------------
    # 获取含有指定字符串的图片路径列表
    def getImgPathContain(self, filteStr):
        if(self.containFilterStrImgPathList):
            return self.containFilterStrImgPathList
        else:
            return '不存在含有 {} 的路径'.format(filteStr)

    # 获取经过清洗后的图片路径列表
    def getCleanedImgsList(self):
        if(self.cleanedImgsListIsChanged):
            return self.cleanedImgsList
        else:
            return '图片路径未经过清洗'

    def run_byWaterMask(self):
        # 水印识别过滤的整个步骤集成
        # 关键词过滤
        self.filterStrList()
        # 过滤掉打不开的图片
        self.filterNoneFile()
        # 尺寸过滤
        self.filterImgBySizeAll()
        # 将经过初步过滤的图片复制到新的目录下
        self.setCopyCleanedImgsList()
        # 水印裁切
        self.classifyImgsListByWaterMask()

    def run_nomaskOp(self):
        # 过滤的整个步骤集成
        # 关键词过滤
        self.filterStrList()
        # 过滤掉打不开的图片
        self.filterNoneFile()
        # 尺寸过滤
        # self.filterImgBySizeAll()
        # 将经过初步过滤的图片复制到新的目录下
        self.setCopyCleanedImgsList()

    def run_hasmaskOp(self):
        # 过滤的整个步骤集成
        # 关键词过滤
        self.filterStrList()
        # 过滤掉打不开的图片
        self.filterNoneFile()

        # 尺寸过滤
        # self.filterImgBySizeAll()
        # 将经过初步过滤的图片复制到新的目录下
        self.setCopyCleanedImgsList()
        # 水印裁切
        # self.cutOffWaterMaskByDir()
        self.classifyImgsListByWaterMask()

    def run_(self):
        # 过滤的整个步骤集成
        # 关键词过滤
        self.filterStrList()
        # 过滤掉打不开的图片
        self.filterNoneFile()
        # 将经过初步过滤的图片复制到新的目录下
        self.setCopyCleanedImgsList()









from customFunction__.Cracker.base_slidercrack import Base_Slidercrack
import time

class Slidercheck_Douyin(Base_Slidercrack):
    def __init__(self, captchaDstDirPath, driverPath='E:\Projects\webDriver\chrome\\chromedriver.exe'):
        Base_Slidercrack.__init__(self, driverPath=driverPath)
        self.captchaDstDirPath = captchaDstDirPath  # 滑块验证图片保存的路径

    def get_selector(self, *browser):
        if (browser == ()):
                # 无指定浏览器
                self.sliderTargetSelector = self.browser.find_element_by_xpath(
                    "//img[@id='captcha-verify-image']"
                )  # 背景
                self.sliderTemplateSelector = self.browser.find_element_by_xpath(
                    "//img[@class='captcha_verify_img_slide react-draggable sc-VigVT ggNWOG']"
                )  # 滑块
        else:
                self.sliderTargetSelector = browser[0].find_element_by_xpath(
                    "//img[@id='captcha-verify-image']"
                )  # 背景
                self.sliderTemplateSelector = browser[0].find_element_by_xpath(
                    "//img[@class='captcha_verify_img_slide react-draggable sc-VigVT ggNWOG']"
                )  # 滑块

    def handle_SlideCheck(self, *browser):
        '''
            处理指定browser的头条滑块验证
        :param browser: 指定的browser
        :return:
        '''
        while(1):
            if (browser == ()):
                self.get_selector()
                # 无指定浏览器
                targetWidth = self.download_slideImgs(self.browser)  # 滑块图片下载器 返回目标网页显示的宽
                movement = self.getMovement_sliderCheck_method1(
                    self.captchaDstDirPath + 'template.png',
                    self.captchaDstDirPath + 'target.png',
                    targetWidth
                )  # 验证码处理器
                print("滑动的实际距离：", movement)
                # 滑动滑块
                # self.move_sliderTemplate(int(movement), self.sliderTemplateSelector)
                self.move_sliderTemplate_likeHuman(int(movement), templateSelector=self.sliderTemplateSelector)
            else:
                self.get_selector(browser[0])
                targetWidth = self.download_slideImgs(browser)  # 滑块图片下载器 返回目标网页显示的宽
                movement = self.getMovement_sliderCheck_method1(
                    self.captchaDstDirPath + 'template.png',
                    self.captchaDstDirPath + 'target.png',
                    targetWidth
                )
                print("滑动的实际距离：", movement)
                # 滑动滑块
                # self.move_sliderTemplate(int(movement))
                self.move_sliderTemplate_likeHuman(int(movement), browser[0], templateSelector=self.sliderTemplateSelector)

            # 滑完一会查看是否需要验证，不需要则滑块验证通过，否则重新执行该方法
            time.sleep(2)
            try:
                if (browser == ()):
                    # 无指定浏览器
                    sliderTargetSelector = self.browser.find_element_by_xpath(
                        "//img[@id='captcha-verify-image']"
                    )  # 背景
                else:
                    sliderTargetSelector = browser.find_element_by_xpath(
                        "//img[@id='captcha-verify-image']"
                    )  # 背景

            except Exception as e:
                print("验证通过")
                break
            # 验证通过后输出cookie
        return self.get_Obj_allCookies()

    def download_slideImgs(self, *browser):
        '''
        下载滑块以及滑块背景用于识别位置
        :return: targetWidth 背景图在网页中的宽度 用于步长计算
        '''

            # 背景 target
            # 滑块 template
        try:
            targetImgUrl = self.sliderTargetSelector.get_attribute('src')  # 背景图url
            templateImgUrl = self.sliderTemplateSelector.get_attribute('src')  # 滑块url
            targetWidth = self.sliderTargetSelector.size['width']  # 背景图片宽
            templateWidth = self.sliderTemplateSelector.size['width']  # 滑块宽
            # 下载模板图片——滑块
            dstDirPath = self.captchaDstDirPath
            self.downloader.download_img(templateImgUrl, 'template', dstDirPath)
            # 下载背景图片
            self.downloader.download_img(targetImgUrl, 'target', dstDirPath)
            return targetWidth

        except Exception as e:
            print("无滑块出现，下载失败请刷新浏览器重新操作")
            return ''


    '''头条的滑块验证功能'''
    def slidecrack_run(self):
        '''
        :param browser 待操作的浏览器
        :param saved_path 图片保存的路径
        '''
        time.sleep(3)
        # 获取滑块
        self.get_selector()
        # 移动滑块
        self.handle_SlideCheck()

